@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,9 +31,10 @@
 #include <sys/socket.h>
 #endif
 #include <fcntl.h>
+#ifndef G_OS_WIN32
 #include <sys/mman.h>
 #include <sys/wait.h>
-
+#endif
 #include <gtk/gtk.h>
 #include <log.h>
 #include "etpan-thread-manager.h"
@@ -84,6 +85,11 @@ static gboolean thread_manager_event(GIOChannel * source,
 	return TRUE;
 }
 
+static void imap_logger_noop(int direction, const char * str, size_t size) 
+{
+	/* inhibit logging */
+}
+
 static void imap_logger_cmd(int direction, const char * str, size_t size) 
 {
 	gchar *buf;
@@ -125,7 +131,7 @@ static void imap_logger_fetch(int direction, const char * str, size_t size)
 	gchar **lines;
 	int i = 0;
 
-	if (size > 8192) {
+	if (size > 128 && !direction) {
 		log_print(LOG_PROTOCOL, "IMAP4%c [FETCH data - %zd bytes]\n", direction?'>':'<', size);
 		return;
 	}
@@ -191,7 +197,7 @@ static void imap_logger_uid(int direction, const char * str, size_t size)
 		else {
 			gchar tmp[64];
 			strncpy2(tmp, lines[i], 63);
-			log_print(LOG_PROTOCOL, "IMAP4%c %s[... - %zd bytes more]\n", direction?'>':'<', tmp,
+			log_print(LOG_PROTOCOL, "IMAP4%c %s[... - %d bytes more]\n", direction?'>':'<', tmp,
 				  llen-64);
 		}
 		i++;
@@ -242,7 +248,7 @@ static void imap_logger_append(int direction, const char * str, size_t size)
 }
 
 #define ETPAN_DEFAULT_NETWORK_TIMEOUT 60
-static gboolean etpan_skip_ssl_cert_check = FALSE;
+gboolean etpan_skip_ssl_cert_check = FALSE;
 extern void mailsasl_ref(void);
 
 void imap_main_init(gboolean skip_ssl_cert_check)
@@ -282,6 +288,9 @@ void imap_main_set_timeout(int sec)
 void imap_main_done(void)
 {
 	etpan_thread_manager_stop(thread_manager);
+#if defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__)
+	return;
+#endif
 	etpan_thread_manager_join(thread_manager);
 	
 	g_source_remove(thread_manager_signal);
@@ -367,6 +376,11 @@ static mailimap * get_imap(Folder * folder)
 	return imap;
 }
 
+static gboolean cb_show_error(gpointer data)
+{
+	mainwindow_show_error();
+	return FALSE;
+}
 
 static void generic_cb(int cancelled, void * result, void * callback_data)
 {
@@ -379,7 +393,7 @@ static void generic_cb(int cancelled, void * result, void * callback_data)
 	    op->imap->imap_response_info->rsp_alert) {
 		log_error(LOG_PROTOCOL, "IMAP4< Alert: %s\n", 
 			op->imap->imap_response_info->rsp_alert);
-		mainwindow_show_error();
+		g_timeout_add(10, cb_show_error, NULL);
 	} 
 	op->finished = 1;
 }
@@ -501,9 +515,9 @@ static int etpan_certificate_check(const unsigned char *certificate, int len, vo
 		g_warning("no cert presented.\n");
 		return 0;
 	}
-	cert = d2i_X509(NULL, &certificate, len);
+	cert = d2i_X509(NULL, (const unsigned char **)&certificate, len);
 	if (cert == NULL) {
-		g_warning("can't get cert\n");
+		g_warning("IMAP: can't get cert\n");
 		return 0;
 	} else if (ssl_certificate_check(cert, NULL,
 		(gchar *)param->server, (gushort)param->port) == TRUE) {
@@ -513,9 +527,33 @@ static int etpan_certificate_check(const unsigned char *certificate, int len, vo
 		X509_free(cert);
 		return -1;
 	}
-#else
-	return 0;
+#elif USE_GNUTLS
+	struct connect_param *param = (struct connect_param *)data;
+	gnutls_x509_crt cert = NULL;
+	gnutls_datum tmp;
+	
+	if (certificate == NULL || len < 0) {
+		g_warning("no cert presented.\n");
+		return 0;
+	}
+	
+	tmp.data = malloc(len);
+	memcpy(tmp.data, certificate, len);
+	tmp.size = len;
+	gnutls_x509_crt_init(&cert);
+	if (gnutls_x509_crt_import(cert, &tmp, GNUTLS_X509_FMT_DER) < 0) {
+		g_warning("IMAP: can't get cert\n");
+		return 0;
+	} else if (ssl_certificate_check(cert, (guint)-1, NULL,
+		(gchar *)param->server, (gushort)param->port) == TRUE) {
+		gnutls_x509_crt_deinit(cert);
+		return 0;
+	} else {
+		gnutls_x509_crt_deinit(cert);
+		return -1;
+	}
 #endif
+	return 0;
 }
 
 static void connect_ssl_run(struct etpan_thread_op * op)
@@ -867,6 +905,8 @@ static void login_run(struct etpan_thread_op * op)
 #endif
 	
 	result->error = r;
+	if (param->imap->imap_response)
+		imap_logger_cmd(0, param->imap->imap_response, strlen(param->imap->imap_response));
 	debug_print("imap login run - end %i\n", r);
 }
 
@@ -999,7 +1039,12 @@ static void noop_run(struct etpan_thread_op * op)
 	debug_print("imap noop run - end %i\n", r);
 }
 
-int imap_threaded_noop(Folder * folder, unsigned int * p_exists)
+int imap_threaded_noop(Folder * folder, unsigned int * p_exists, 
+		       unsigned int *p_recent, 
+		       unsigned int *p_expunge,
+		       unsigned int *p_unseen,
+		       unsigned int *p_uidnext,
+		       unsigned int *p_uidval)
 {
 	struct noop_param param;
 	struct noop_result result;
@@ -1012,14 +1057,28 @@ int imap_threaded_noop(Folder * folder, unsigned int * p_exists)
 
 	threaded_run(folder, &param, &result, noop_run);
 	
-	if (imap && imap->imap_selection_info != NULL) {
+	if (result.error == 0 && imap && imap->imap_selection_info != NULL) {
 		* p_exists = imap->imap_selection_info->sel_exists;
-	}
-	else {
+		* p_recent = imap->imap_selection_info->sel_recent;
+		* p_unseen = imap->imap_selection_info->sel_unseen;
+		* p_uidnext = imap->imap_selection_info->sel_uidnext;
+		* p_uidval = imap->imap_selection_info->sel_uidvalidity;
+	} else {
 		* p_exists = 0;
+		* p_recent = 0;
+		* p_unseen = 0;
+		* p_uidnext = 0;
+		* p_uidval = 0;
 	}
-	
-	debug_print("imap noop - end\n");
+	if (result.error == 0 && imap && imap->imap_response_info != NULL &&
+	    imap->imap_response_info->rsp_expunged != NULL) {
+		* p_expunge = clist_count(imap->imap_response_info->rsp_expunged);
+	} else {
+		* p_expunge = 0;
+	}	
+	debug_print("imap noop - end [EXISTS %d RECENT %d EXPUNGE %d UNSEEN %d UIDNEXT %d UIDVAL %d]\n",
+		*p_exists, *p_recent, *p_expunge, *p_unseen,
+		*p_uidnext, *p_uidval);
 	
 	return result.error;
 }
@@ -1267,12 +1326,13 @@ static void select_run(struct etpan_thread_op * op)
 
 int imap_threaded_select(Folder * folder, const char * mb,
 			 gint * exists, gint * recent, gint * unseen,
-			 guint32 * uid_validity)
+			 guint32 * uid_validity,gint *can_create_flags,
+			 GSList **ok_flags)
 {
 	struct select_param param;
 	struct select_result result;
 	mailimap * imap;
-	
+
 	debug_print("imap select - begin\n");
 	
 	imap = get_imap(folder);
@@ -1291,13 +1351,96 @@ int imap_threaded_select(Folder * folder, const char * mb,
 	* recent = imap->imap_selection_info->sel_recent;
 	* unseen = imap->imap_selection_info->sel_unseen;
 	* uid_validity = imap->imap_selection_info->sel_uidvalidity;
-	
+	* can_create_flags = FALSE;
+
+	if (imap->imap_selection_info->sel_perm_flags) {
+		GSList *t_flags = NULL;
+		clistiter *cur =
+			clist_begin(imap->imap_selection_info->sel_perm_flags);
+		for (; cur; cur = clist_next(cur)) {
+			struct mailimap_flag_perm *flag = (struct mailimap_flag_perm *)clist_content(cur);
+			if (flag->fl_type == MAILIMAP_FLAG_PERM_ALL)
+				*can_create_flags = TRUE;
+			else if (flag->fl_flag && 
+					flag->fl_flag->fl_type == 6 &&
+					!strcmp(flag->fl_flag->fl_data.fl_extension, "*"))
+				*can_create_flags = TRUE; 
+			if (ok_flags) {
+				MsgPermFlags c_flag = 0;
+				switch (flag->fl_flag->fl_type) {
+				case MAILIMAP_FLAG_ANSWERED:
+					c_flag = IMAP_FLAG_ANSWERED;
+					break;
+				case MAILIMAP_FLAG_FLAGGED:
+					c_flag = IMAP_FLAG_FLAGGED;
+					break;
+				case MAILIMAP_FLAG_DELETED:
+					c_flag = IMAP_FLAG_DELETED;
+					break;
+				case MAILIMAP_FLAG_DRAFT:
+					c_flag = IMAP_FLAG_DRAFT;
+					break;
+				case MAILIMAP_FLAG_SEEN:
+					c_flag = IMAP_FLAG_SEEN;
+					break;
+				case MAILIMAP_FLAG_KEYWORD:
+					if (!strcasecmp(flag->fl_flag->fl_data.fl_keyword, "$Forwarded"))
+						c_flag = IMAP_FLAG_FORWARDED;
+					break;
+				default:
+					break;
+				}
+				if (c_flag != 0) {
+					t_flags = g_slist_prepend(t_flags, 
+						GUINT_TO_POINTER(c_flag));
+				}
+			}
+		}
+		if (ok_flags)
+			*ok_flags = t_flags;
+	}
 	debug_print("imap select - end\n");
 	
 	return result.error;
 }
 
+static void close_run(struct etpan_thread_op * op)
+{
+	struct select_param * param;
+	struct select_result * result;
+	int r;
+	
+	param = op->param;
+	result = op->result;
 
+	CHECK_IMAP();
+
+	r = mailimap_close(param->imap);
+	
+	result->error = r;
+	debug_print("imap close run - end %i\n", r);
+}
+
+int imap_threaded_close(Folder * folder)
+{
+	struct select_param param;
+	struct select_result result;
+	mailimap * imap;
+	
+	debug_print("imap close - begin\n");
+	
+	imap = get_imap(folder);
+	param.imap = imap;
+	
+	threaded_run(folder, &param, &result, close_run);
+	
+	if (result.error != MAILIMAP_NO_ERROR)
+		return result.error;
+	
+	debug_print("imap close - end\n");
+	
+	return result.error;
+}
 
 struct examine_param {
 	mailimap * imap;
@@ -1461,6 +1604,14 @@ static void search_run(struct etpan_thread_op * op)
 							  NULL, NULL, NULL, NULL, NULL,
 							  NULL, 0, NULL, NULL, NULL);
 		break;
+	case IMAP_SEARCH_TYPE_FORWARDED:
+		search_type_key = mailimap_search_key_new(MAILIMAP_SEARCH_KEY_KEYWORD,
+							  NULL, NULL, NULL, NULL, NULL,
+							  strdup("$Forwarded"), NULL, NULL, NULL, NULL,
+							  NULL, NULL, NULL, NULL, 0,
+							  NULL, NULL, NULL, NULL, NULL,
+							  NULL, 0, NULL, NULL, NULL);
+		break;
 	}
 	
 	if (search_type_key != NULL) {
@@ -1617,9 +1768,12 @@ static int imap_get_messages_list(mailimap * imap,
 		goto free_fetch_type;
 	}
 
+	mailstream_logger = imap_logger_fetch;
+	
 	r = mailimap_uid_fetch(imap, set,
 			       fetch_type, &fetch_result);
 
+	mailstream_logger = imap_logger_cmd;
 	mailimap_fetch_type_free(fetch_type);
 	mailimap_set_free(set);
 
@@ -1670,7 +1824,7 @@ static void fetch_uid_run(struct etpan_thread_op * op)
 	CHECK_IMAP();
 
 	fetch_result = NULL;
-	mailstream_logger = NULL;
+	mailstream_logger = imap_logger_noop;
 	log_print(LOG_PROTOCOL, "IMAP4- [fetching UIDs...]\n");
 
 	r = imap_get_messages_list(param->imap, param->first_index,
@@ -1725,7 +1879,7 @@ void imap_fetch_uid_list_free(carray * uid_list)
 
 
 
-static int imap_flags_to_flags(struct mailimap_msg_att_dynamic * att_dyn);
+static int imap_flags_to_flags(struct mailimap_msg_att_dynamic * att_dyn, GSList **tags);
 
 static int
 result_to_uid_flags_list(clist * fetch_result, carray ** result)
@@ -1734,7 +1888,8 @@ result_to_uid_flags_list(clist * fetch_result, carray ** result)
 	int r;
 	int res;
 	carray * tab;
-	
+	GSList *tags = NULL;
+
 	tab = carray_new(128);
 	if (tab == NULL) {
 		res = MAILIMAP_ERROR_MEMORY;
@@ -1750,6 +1905,8 @@ result_to_uid_flags_list(clist * fetch_result, carray ** result)
 		int flags;
 		int * pflags;
 		
+		tags = NULL;
+
 		msg_att = clist_content(cur);
 		
 		uid = 0;
@@ -1760,7 +1917,7 @@ result_to_uid_flags_list(clist * fetch_result, carray ** result)
 		if (att_dyn == NULL)
 			continue;
 		
-		flags = imap_flags_to_flags(att_dyn);
+		flags = imap_flags_to_flags(att_dyn, &tags);
 		
 		puid = malloc(sizeof(* puid));
 		if (puid == NULL) {
@@ -1787,6 +1944,12 @@ result_to_uid_flags_list(clist * fetch_result, carray ** result)
 			res = MAILIMAP_ERROR_MEMORY;
 			goto free_list;
 		}
+		r = carray_add(tab, tags, NULL);
+		if (r < 0) {
+			free(pflags);
+			res = MAILIMAP_ERROR_MEMORY;
+			goto free_list;
+		}
 	}
 		
 	* result = tab;
@@ -1795,6 +1958,8 @@ result_to_uid_flags_list(clist * fetch_result, carray ** result)
   
  free_list:
 	imap_fetch_uid_flags_list_free(tab);
+	slist_free_strings(tags);
+	g_slist_free(tags);
  err:
 	return res;
 }
@@ -1849,9 +2014,12 @@ static int imap_get_messages_flags_list(mailimap * imap,
 		goto free_fetch_type;
 	}
 
+	mailstream_logger = imap_logger_fetch;
+	
 	r = mailimap_uid_fetch(imap, set,
 			       fetch_type, &fetch_result);
 
+	mailstream_logger = imap_logger_cmd;
 	mailimap_fetch_type_free(fetch_type);
 	mailimap_set_free(set);
 
@@ -1912,7 +2080,7 @@ int imap_threaded_fetch_uid_flags(Folder * folder, uint32_t first_index,
 	param.imap = imap;
 	param.first_index = first_index;
 	
-	mailstream_logger = NULL;
+	mailstream_logger = imap_logger_noop;
 	log_print(LOG_PROTOCOL, "IMAP4- [fetching flags...]\n");
 
 	threaded_run(folder, &param, &result, fetch_uid_flags_run);
@@ -1935,11 +2103,13 @@ void imap_fetch_uid_flags_list_free(carray * uid_flags_list)
 {
 	unsigned int i;
 	
-	for(i = 0 ; i < carray_count(uid_flags_list) ; i ++) {
+	for(i = 0 ; i < carray_count(uid_flags_list) ; i += 3) {
 		void * data;
 		
 		data = carray_get(uid_flags_list, i);
 		free(data);
+		data = carray_get(uid_flags_list, i + 1);
+		free(data);
 	}
 	carray_free(uid_flags_list);
 }
@@ -2093,8 +2263,11 @@ static int imap_fetch_header(mailimap * imap,
     goto free_fetch_att;
   }
 
+  mailstream_logger = imap_logger_fetch;
+  
   r = mailimap_uid_fetch(imap, set, fetch_type, &fetch_result);
   
+  mailstream_logger = imap_logger_cmd;
   mailimap_fetch_type_free(fetch_type);
   mailimap_set_free(set);
 
@@ -2201,14 +2374,15 @@ static void fetch_content_run(struct etpan_thread_op * op)
 		}
 		
 		r = fwrite(content, 1, content_size, f);
-		if (r == 0) {
+		if (r < content_size) {
+			result->error = MAILIMAP_ERROR_FETCH;
 			goto fclose;
 		}
 		
 		r = fclose(f);
 		if (r == EOF) {
-			g_unlink(param->filename);
-			goto close;
+			result->error = MAILIMAP_ERROR_FETCH;
+			goto unlink;
 		}
 		goto free;
 		
@@ -2221,11 +2395,13 @@ static void fetch_content_run(struct etpan_thread_op * op)
 		g_unlink(param->filename);
 	
 	free:
+		/* mmap_string_unref is a simple free in libetpan
+		 * when it has MMAP_UNAVAILABLE defined */
 		if (mmap_string_unref(content) != 0)
 			free(content);
 	}
 	
-	debug_print("imap fetch_content run - end %i\n", r);
+	debug_print("imap fetch_content run - end %i\n", result->error);
 }
 
 int imap_threaded_fetch_content(Folder * folder, uint32_t msg_index,
@@ -2256,12 +2432,13 @@ int imap_threaded_fetch_content(Folder * folder, uint32_t msg_index,
 
 
 
-static int imap_flags_to_flags(struct mailimap_msg_att_dynamic * att_dyn)
+static int imap_flags_to_flags(struct mailimap_msg_att_dynamic * att_dyn, GSList **s_tags)
 {
 	int flags;
 	clist * flag_list;
 	clistiter * cur;
-	
+	GSList *tags = NULL;
+
 	flags = MSG_UNREAD;
 	
 	flag_list = att_dyn->att_list;
@@ -2290,10 +2467,17 @@ static int imap_flags_to_flags(struct mailimap_msg_att_dynamic * att_dyn)
 				flags &= ~MSG_UNREAD;
 				flags &= ~MSG_NEW;
 				break;
+			case MAILIMAP_FLAG_KEYWORD:
+				if (!strcasecmp(flag_fetch->fl_flag->fl_data.fl_keyword, "$Forwarded"))
+					flags |= MSG_FORWARDED;
+				else if (s_tags)
+					tags = g_slist_prepend(tags, g_strdup(flag_fetch->fl_flag->fl_data.fl_keyword));
+				break;
 			}
 		}
 	}
-	
+	if (s_tags)
+		*s_tags = tags;
 	return flags;
 }
 
@@ -2359,14 +2543,14 @@ static int imap_get_msg_att_info(struct mailimap_msg_att * msg_att,
 }
 
 static struct imap_fetch_env_info *
-fetch_to_env_info(struct mailimap_msg_att * msg_att)
+fetch_to_env_info(struct mailimap_msg_att * msg_att, GSList **tags)
 {
 	struct imap_fetch_env_info * info;
 	uint32_t uid;
 	char * headers;
 	size_t size;
 	struct mailimap_msg_att_dynamic * att_dyn;
-	
+
 	imap_get_msg_att_info(msg_att, &uid, &headers, &size,
 			      &att_dyn);
 	
@@ -2376,7 +2560,7 @@ fetch_to_env_info(struct mailimap_msg_att * msg_att)
 	info->uid = uid;
 	info->headers = strdup(headers);
 	info->size = size;
-	info->flags = imap_flags_to_flags(att_dyn);
+	info->flags = imap_flags_to_flags(att_dyn, tags);
 	
 	return info;
 }
@@ -2388,21 +2572,23 @@ imap_fetch_result_to_envelop_list(clist * fetch_result,
 	clistiter * cur;
 	unsigned int i;
 	carray * env_list;
+
 	i = 0;
-  
 	env_list = carray_new(16);
   
 	for(cur = clist_begin(fetch_result) ; cur != NULL ;
 	    cur = clist_next(cur)) {
 		struct mailimap_msg_att * msg_att;
 		struct imap_fetch_env_info * env_info;
-    
+	    	GSList *tags = NULL;
+
 		msg_att = clist_content(cur);
 
-		env_info = fetch_to_env_info(msg_att);
+		env_info = fetch_to_env_info(msg_att, &tags);
 		if (!env_info)
 			return MAILIMAP_ERROR_MEMORY;
 		carray_add(env_list, env_info, NULL);
+		carray_add(env_list, tags, NULL);
 	}
   
 	* p_env_list = env_list;
@@ -2494,8 +2680,11 @@ imap_get_envelopes_list(mailimap * imap, struct mailimap_set * set,
 	else
 		r = imap_add_header_fetch_att(fetch_type);
 	
+	mailstream_logger = imap_logger_fetch;
+	
 	r = mailimap_uid_fetch(imap, set, fetch_type, &fetch_result);
 	
+	mailstream_logger = imap_logger_cmd;
 	switch (r) {
 	case MAILIMAP_NO_ERROR:
 		break;
@@ -2609,7 +2798,7 @@ void imap_fetch_env_free(carray * env_list)
 {
 	unsigned int i;
 	
-	for(i = 0 ; i < carray_count(env_list) ; i ++) {
+	for(i = 0 ; i < carray_count(env_list) ; i += 2) {
 		struct imap_fetch_env_info * env_info;
 		
 		env_info = carray_get(env_list, i);
@@ -2642,8 +2831,10 @@ static void append_run(struct etpan_thread_op * op)
 	int r;
 	char * data;
 	size_t size;
+#ifndef G_OS_WIN32
 	struct stat stat_buf;
 	int fd;
+#endif
 	guint32 uid = 0, val = 0;
 	
 	param = op->param;
@@ -2651,6 +2842,7 @@ static void append_run(struct etpan_thread_op * op)
 	
 	CHECK_IMAP();
 
+#ifndef G_OS_WIN32
 	r = stat(param->filename, &stat_buf);
 	if (r < 0) {
 		result->error = MAILIMAP_ERROR_APPEND;
@@ -2670,7 +2862,14 @@ static void append_run(struct etpan_thread_op * op)
 		result->error = MAILIMAP_ERROR_APPEND;
 		return;
 	}
-	
+#else
+	data = file_read_to_str_no_recode(param->filename);
+	if (data == NULL) {
+		result->error = MAILIMAP_ERROR_APPEND;
+		return;
+	}
+	size = strlen(data);
+#endif
 	mailstream_logger = imap_logger_append;
 	
 	r = mailimap_uidplus_append(param->imap, param->mailbox,
@@ -2679,8 +2878,12 @@ static void append_run(struct etpan_thread_op * op)
 
 	mailstream_logger = imap_logger_cmd;
 	
+#ifndef G_OS_WIN32
 	munmap(data, size);
 	close(fd);
+#else
+	g_free(data);
+#endif
 	
 	result->error = r;
 	result->uid = uid;
@@ -2887,7 +3090,7 @@ int imap_threaded_store(Folder * folder, struct mailimap_set * set,
 
 
 #define ENV_BUFFER_SIZE 512
-
+#ifndef G_OS_WIN32
 static void do_exec_command(int fd, const char * command,
 			    const char * servername, uint16_t port)
 {
@@ -3086,6 +3289,7 @@ int imap_threaded_connect_cmd(Folder * folder, const char * command,
 	
 	return result.error;
 }
+#endif /* G_OS_WIN32 */
 
 void imap_threaded_cancel(Folder * folder)
 {
