@@ -2156,6 +2156,9 @@ void folderview_close_opened(FolderView *folderview)
 			folder_item_close(olditem);
 			main_window_cursor_normal(folderview->mainwin);
 			STATUSBAR_POP(folderview->mainwin);
+			if (olditem->folder->klass->item_closed)
+				olditem->folder->klass->item_closed(olditem);
+
 		}
 	}
 
