@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,14 +48,15 @@ typedef struct _PrefsTreeNode PrefsTreeNode;
 struct _PrefsWindow
 {
 	GtkWidget *window;
-	GtkWidget *table1;
+	GtkWidget *vbox;
+	GtkWidget *paned;
 	GtkWidget *scrolledwindow1;
 	GtkWidget *scrolledwindow2;
 	GtkWidget *tree_view;
 	GtkWidget *table2;
 	GtkWidget *pagelabel;
 	GtkWidget *labelframe;
-	GtkWidget *frame;
+	GtkWidget *vbox2;
 	GtkWidget *notebook;
 	GtkWidget *confirm_area;
 	GtkWidget *ok_btn;
@@ -63,6 +64,8 @@ struct _PrefsWindow
 	GtkWidget *apply_btn;
 	gint *save_width;
 	gint *save_height;
+	PrefsCloseCallbackFunc open_cb;
+	PrefsCloseCallbackFunc close_cb;
 
 	GtkWidget *empty_page;
 
@@ -129,11 +132,44 @@ static void close_all_pages(GSList *prefs_pages)
 	}	
 }
 
+#ifdef MAEMO
+static void prefs_show_sections(PrefsWindow *prefswindow)
+{
+	gint max;
+	GtkWidget *paned = prefswindow->paned;
+	
+        g_object_get (G_OBJECT(paned),
+                        "max-position",
+                        &max, NULL);
+
+	gtk_widget_show(gtk_paned_get_child1(GTK_PANED(paned)));
+	gtk_widget_hide(gtk_paned_get_child2(GTK_PANED(paned)));
+	gtk_paned_set_position(GTK_PANED(paned), max);
+}
+
+static void prefs_show_page(PrefsWindow *prefswindow)
+{
+	gint min;
+	GtkWidget *paned = prefswindow->paned;
+	
+        g_object_get (G_OBJECT(paned),
+                        "min-position",
+                        &min, NULL);
+
+	gtk_widget_hide(gtk_paned_get_child1(GTK_PANED(paned)));
+	gtk_widget_show(gtk_paned_get_child2(GTK_PANED(paned)));
+	gtk_paned_set_position(GTK_PANED(paned), min);
+}
+#endif
+
 static void apply_button_clicked(GtkButton *button, gpointer user_data)
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
 	save_all_pages(prefswindow->prefs_pages);
+#ifdef MAEMO
+	prefs_show_sections(prefswindow);
+#endif
 }
 
 static void close_prefs_window(PrefsWindow *prefswindow)
@@ -142,6 +178,9 @@ static void close_prefs_window(PrefsWindow *prefswindow)
 
 	close_all_pages(prefswindow->prefs_pages);
 
+	if (prefswindow->close_cb)
+		prefswindow->close_cb(GTK_WINDOW(prefswindow->window));
+
 	gtk_widget_destroy(prefswindow->window);
 	g_slist_free(prefswindow->prefs_pages);
 	if(prefswindow->func != NULL)
@@ -158,22 +197,26 @@ static void ok_button_clicked(GtkButton *button, gpointer user_data)
 		close_prefs_window(prefswindow);
 	}		
 }
-
+#ifndef MAEMO
 static void cancel_button_clicked(GtkButton *button, gpointer user_data)
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
 	close_prefs_window(prefswindow);
 }
-
+#endif
 static gboolean window_closed(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
+#ifdef MAEMO
+	save_all_pages(prefswindow->prefs_pages);
+#endif
 	close_prefs_window(prefswindow);
+
 	return FALSE;
 }
-
+#ifndef MAEMO
 static gboolean prefswindow_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				    PrefsWindow *data)
 {
@@ -200,7 +243,7 @@ static gboolean prefswindow_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	}
 	return FALSE;
 }
-
+#endif
 typedef struct FindNodeByName {
 	const gchar *name;
 	gboolean     found;
@@ -274,9 +317,11 @@ static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages,
 			(GTK_TREE_VIEW(tree_view)));
 	GSList *cur;
 	gint index; /* index in pages list */
+#ifndef MAEMO
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
-	
+#endif
+
 	for (cur = prefs_pages, index = 0; cur != NULL; cur = g_slist_next(cur), index++) {
 		PrefsPage *page = (PrefsPage *)cur->data;
 		FindNodeByName find_name;
@@ -354,6 +399,7 @@ static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages,
 		prefswindow_build_all_pages(prefswindow, prefs_pages);
 
 	/* select first one or its first child if necessary */
+#ifndef MAEMO
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
 	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
 		if (activate_child && gtk_tree_model_iter_has_child(GTK_TREE_MODEL(store), &iter)) {
@@ -363,17 +409,22 @@ static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages,
 		}
 		gtk_tree_selection_select_iter(selection, &iter);
 	}
+#endif
 }
 
 void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 							 gpointer data, GtkDestroyNotify func,
 							 gint *save_width, gint *save_height,
-							 gboolean preload_pages, gboolean activate_child)
+							 gboolean preload_pages, gboolean activate_child,
+							 PrefsOpenCallbackFunc open_cb,
+							 PrefsCloseCallbackFunc close_cb)
 {
 	PrefsWindow *prefswindow;
 	gint x = gdk_screen_width();
 	gint y = gdk_screen_height();
 	static GdkGeometry geometry;
+	GtkAdjustment *adj;
+	GtkWidget *tmp;
 
 	prefswindow = g_new0(PrefsWindow, 1);
 
@@ -382,6 +433,8 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	prefswindow->prefs_pages = g_slist_copy(prefs_pages);
 	prefswindow->save_width = save_width;
 	prefswindow->save_height = save_height;
+	prefswindow->open_cb = open_cb;
+	prefswindow->close_cb = close_cb;
 
 	prefswindow->window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefswindow");
 	gtk_window_set_title(GTK_WINDOW(prefswindow->window), title);
@@ -391,31 +444,38 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	gtk_window_set_resizable (GTK_WINDOW(prefswindow->window), TRUE);
 	gtk_container_set_border_width(GTK_CONTAINER(prefswindow->window), 4);
 
-	prefswindow->table1 = gtk_table_new(2, 2, FALSE);
-	gtk_widget_show(prefswindow->table1);
-	gtk_container_add(GTK_CONTAINER(prefswindow->window), prefswindow->table1);
+	prefswindow->vbox = gtk_vbox_new(FALSE, 6);
+	gtk_widget_show(prefswindow->vbox);
+	
+	prefswindow->paned = gtk_hpaned_new();
+	gtk_widget_show(prefswindow->paned);
+
+	gtk_container_add(GTK_CONTAINER(prefswindow->window), prefswindow->vbox);
+
+	gtk_box_pack_start(GTK_BOX(prefswindow->vbox), prefswindow->paned, TRUE, TRUE, 0);
 
 	prefswindow->scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(prefswindow->scrolledwindow1);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow1),
+			GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow1),
 			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	gtk_table_attach(GTK_TABLE(prefswindow->table1), prefswindow->scrolledwindow1,
-			0, 1, 0, 1, GTK_FILL, GTK_FILL | GTK_EXPAND, 2, 2);
+			
+	gtk_paned_add1(GTK_PANED(prefswindow->paned), prefswindow->scrolledwindow1);
 
 	prefswindow->tree_view = prefswindow_tree_view_create(prefswindow);
 	gtk_widget_show(prefswindow->tree_view);
 	gtk_container_add(GTK_CONTAINER(prefswindow->scrolledwindow1), 
 			  prefswindow->tree_view);
 
-	prefswindow->frame = gtk_frame_new(NULL);
-	gtk_widget_show(prefswindow->frame);
-	gtk_frame_set_shadow_type(GTK_FRAME(prefswindow->frame), GTK_SHADOW_IN);
-	gtk_table_attach(GTK_TABLE(prefswindow->table1), prefswindow->frame,
-			1, 2, 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);
+	prefswindow->vbox2 = gtk_vbox_new(FALSE, 2);
+	gtk_widget_show(prefswindow->vbox2);
+
+	gtk_paned_add2(GTK_PANED(prefswindow->paned), prefswindow->vbox2);
 
 	prefswindow->table2 = gtk_table_new(1, 2, FALSE);
 	gtk_widget_show(prefswindow->table2);
-	gtk_container_add(GTK_CONTAINER(prefswindow->frame), prefswindow->table2);
+	gtk_container_add(GTK_CONTAINER(prefswindow->vbox2), prefswindow->table2);
 
 	prefswindow->labelframe = gtk_frame_new(NULL);
 	gtk_widget_show(prefswindow->labelframe);
@@ -441,8 +501,10 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(prefswindow->notebook), FALSE);
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2),
 					prefswindow->notebook);
+	tmp = gtk_bin_get_child(GTK_BIN(prefswindow->scrolledwindow2));
+	gtk_viewport_set_shadow_type(GTK_VIEWPORT(tmp), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(prefswindow->table2), prefswindow->scrolledwindow2,
-			0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 8, 8);
+			0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 4);
 
 	prefswindow->empty_page = gtk_label_new("");
 	gtk_widget_show(prefswindow->empty_page);
@@ -451,24 +513,43 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	prefswindow_build_tree(prefswindow->tree_view, prefs_pages, prefswindow,
 							preload_pages, activate_child);
 
+	if (open_cb)
+		open_cb(GTK_WINDOW(prefswindow->window));
+
 	gtk_widget_grab_focus(prefswindow->tree_view);
 
+#ifndef MAEMO
 	gtkut_stock_button_set_create(&prefswindow->confirm_area,
 				      &prefswindow->apply_btn,	GTK_STOCK_APPLY,
 				      &prefswindow->cancel_btn,	GTK_STOCK_CANCEL,
 				      &prefswindow->ok_btn,	GTK_STOCK_OK);
-
+#else
+	gtkut_stock_button_set_create(&prefswindow->confirm_area,
+				      &prefswindow->apply_btn,	GTK_STOCK_APPLY,
+				      &prefswindow->ok_btn,	GTK_STOCK_CLOSE,
+				      NULL,			NULL);
+#endif
 	gtk_widget_show_all(prefswindow->confirm_area);
+	gtk_widget_show(prefswindow->vbox);
+	gtk_widget_show(prefswindow->scrolledwindow1);
+	gtk_widget_show(prefswindow->scrolledwindow2);
 
-	gtk_table_attach(GTK_TABLE(prefswindow->table1), prefswindow->confirm_area,
-			0, 2, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 2);
+	gtk_box_pack_start(GTK_BOX(prefswindow->vbox), prefswindow->confirm_area, FALSE, FALSE, 0);
 
+#ifndef MAEMO
 	g_signal_connect(G_OBJECT(prefswindow->ok_btn), "clicked", 
 			 G_CALLBACK(ok_button_clicked), prefswindow);
 	g_signal_connect(G_OBJECT(prefswindow->cancel_btn), "clicked", 
 			 G_CALLBACK(cancel_button_clicked), prefswindow);
 	g_signal_connect(G_OBJECT(prefswindow->apply_btn), "clicked", 
 			 G_CALLBACK(apply_button_clicked), prefswindow);
+#else
+	g_signal_connect(G_OBJECT(prefswindow->ok_btn), "clicked", 
+			 G_CALLBACK(ok_button_clicked), prefswindow);
+	g_signal_connect(G_OBJECT(prefswindow->apply_btn), "clicked", 
+			 G_CALLBACK(apply_button_clicked), prefswindow);
+#endif
+
 	g_signal_connect(G_OBJECT(prefswindow->window), "delete_event", 
 			 G_CALLBACK(window_closed), prefswindow);
 
@@ -502,17 +583,34 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 					    *(prefswindow->save_height));
 	}
 
+#ifdef MAEMO
+	prefs_show_sections(prefswindow);
+#endif
 	gtk_widget_show(prefswindow->window);
 #ifdef MAEMO
 	maemo_window_full_screen_if_needed(GTK_WINDOW(prefswindow->window));
 #endif
+	adj = gtk_scrolled_window_get_vadjustment(
+			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow1));
+	gtk_adjustment_set_value(adj, adj->lower);
+	gtk_adjustment_changed(adj);
+	adj = gtk_scrolled_window_get_vadjustment(
+			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
+	gtk_adjustment_set_value(adj, adj->lower);
+	gtk_adjustment_changed(adj);
+	adj = gtk_scrolled_window_get_hadjustment(
+			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
+	gtk_adjustment_set_value(adj, adj->lower);
+	gtk_adjustment_changed(adj);
 }
 
 void prefswindow_open(const gchar *title, GSList *prefs_pages, gpointer data,
-					 gint *save_width, gint *save_height)
+					 gint *save_width, gint *save_height,
+					 PrefsOpenCallbackFunc open_cb,
+					 PrefsCloseCallbackFunc close_cb)
 {
 	prefswindow_open_full(title, prefs_pages, data, NULL, save_width, save_height,
-						  FALSE, FALSE);
+						  FALSE, FALSE, open_cb, close_cb);
 }
 
 /*!
@@ -593,9 +691,10 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 	GtkTreeIter iter;
 	GtkAdjustment *adj;
 
+#ifndef MAEMO
 	if (currently_selected) 
 		return TRUE;
-
+#endif
 	if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path))
 		return TRUE;
 
@@ -633,11 +732,16 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 
 	adj = gtk_scrolled_window_get_vadjustment(
 			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
-	gtk_adjustment_set_value(adj, 0);
+	gtk_adjustment_set_value(adj, adj->lower);
+	gtk_adjustment_changed(adj);
 	adj = gtk_scrolled_window_get_hadjustment(
 			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
-	gtk_adjustment_set_value(adj, 0);
+	gtk_adjustment_set_value(adj, adj->lower);
+	gtk_adjustment_changed(adj);
 
+#ifdef MAEMO
+	prefs_show_page(prefswindow);
+#endif
 	return TRUE;
 }
 
