@@ -128,7 +128,10 @@ static void set_sensitivity(GtkItemFactory *factory, FolderItem *item)
 	
 	SET_SENS("/Subscriptions/Unsubscribe...",    item->stype == F_NORMAL && folder_item_parent(item) != NULL);
 	SET_SENS("/Subscriptions/Subscribe...",    TRUE);
-	menu_set_active(factory, "/Subscriptions/Show only subscribed folders", item->folder->account->imap_subsonly);
+	if (item->folder && item->folder->account)
+		menu_set_active(factory, 
+			"/Subscriptions/Show only subscribed folders", 
+			item->folder->account->imap_subsonly);
 
 #undef SET_SENS
 }
@@ -269,7 +272,7 @@ static void move_folder_cb(FolderView *folderview, guint action, GtkWidget *widg
 	if (!from_folder || from_folder->folder->klass != imap_get_class())
 		return;
 
-	to_folder = foldersel_folder_sel(from_folder->folder, FOLDER_SEL_MOVE, NULL);
+	to_folder = foldersel_folder_sel(from_folder->folder, FOLDER_SEL_MOVE, NULL, TRUE);
 	if (!to_folder)
 		return;
 	
@@ -549,6 +552,14 @@ static void subscribed_cb(FolderView *folderview, guint action,
 	if (item->folder->account->imap_subsonly == GTK_CHECK_MENU_ITEM(widget)->active)
 		return;
 
+	if (folderview->opened == folderview->selected ||
+	    gtk_ctree_is_ancestor(ctree,
+				  folderview->selected,
+				  folderview->opened)) {
+		summary_clear_all(folderview->summaryview);
+		folderview->opened = NULL;
+	}
+
 	item->folder->account->imap_subsonly = GTK_CHECK_MENU_ITEM(widget)->active;
 	folderview_fast_rescan_tree(item->folder);
 }
