@@ -38,10 +38,16 @@
 #include <sys/types.h>
 #ifdef G_OS_UNIX
 #  include <signal.h>
+#  include <errno.h>
+#  include <fcntl.h>
 #endif
 #ifdef HAVE_LIBSM
 #include <X11/SM/SMlib.h>
-#include <fcntl.h>
+#endif
+#ifdef USE_GNUTLS
+#include <pthread.h>
+#include <gcrypt.h>
+GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #endif
 
 #include "wizard.h"
@@ -213,6 +219,9 @@ static void send_queue			(void);
 static void initial_processing		(FolderItem *item, gpointer data);
 static void quit_signal_handler         (int sig);
 static void install_basic_sighandlers   (void);
+#if (defined linux && defined SIGIO)
+static void install_memory_sighandler   (void);
+#endif
 static void exit_claws			(MainWindow *mainwin);
 
 #ifdef HAVE_NETWORKMANAGER_SUPPORT
@@ -962,6 +971,9 @@ int main(int argc, char *argv[])
 	crash_install_handlers();
 #endif
 	install_basic_sighandlers();
+#if (defined linux && defined SIGIO)
+	install_memory_sighandler();
+#endif
 	sock_init();
 
 	/* check and create unix domain socket for remote operation */
@@ -1055,15 +1067,6 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 #endif	
-	/* parse gtkrc files */
-	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc-2.0",
-			     NULL);
-	gtk_rc_parse(userrc);
-	g_free(userrc);
-	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtk",
-			     G_DIR_SEPARATOR_S, "gtkrc-2.0", NULL);
-	gtk_rc_parse(userrc);
-	g_free(userrc);
 
 #ifdef G_OS_WIN32
 	CHDIR_EXEC_CODE_RETURN_VAL_IF_FAIL(get_home_dir(), 1, win32_close_log(););
@@ -1307,6 +1310,10 @@ int main(int argc, char *argv[])
 	if (claws_crashed())
 		main_window_popup(mainwin);
 
+#ifdef USE_GNUTLS
+	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
+#endif
+
 #ifdef HAVE_LIBETPAN
 	imap_main_init(prefs_common.skip_ssl_cert_check);
 	imap_main_set_timeout(prefs_common.io_timeout_secs);
@@ -2289,6 +2296,57 @@ static void install_basic_sighandlers()
 #endif /* !G_OS_WIN32 */
 }
 
+#if (defined linux && defined SIGIO)
+static int mem_notify_fd = 0;
+
+static gboolean clean_caches(gpointer unused)
+{
+	if (static_mainwindow && static_mainwindow->lock_count > 0)
+		return TRUE;
+	debug_print("/dev/mem_notify: callback: Freeing some memory now!\n");
+	folder_clean_cache_memory_force();
+	return FALSE;
+}
+
+static void memory_signal_handler(int sig)
+{
+	debug_print("/dev/mem_notify: Kernel says we should free up some memory!\n");
+	g_timeout_add(10, clean_caches, NULL); 
+}
+
+static void install_memory_sighandler()
+{
+	sigset_t    mask;
+	struct sigaction act;
+	int flags;
+
+	mem_notify_fd = open("/dev/mem_notify", O_RDONLY|O_NONBLOCK);
+	if (mem_notify_fd == -1) {
+		debug_print("/dev/mem_notify not available (%s)\n", 
+			strerror(errno));
+		return;
+	}
+	
+	fcntl(mem_notify_fd, F_SETOWN, getpid());
+	flags = fcntl(mem_notify_fd, F_GETFL);
+	fcntl(mem_notify_fd, flags|FASYNC);
+
+	sigemptyset(&mask);
+
+	sigaddset(&mask, SIGIO);
+
+	act.sa_handler = memory_signal_handler;
+	act.sa_mask    = mask;
+	act.sa_flags   = 0;
+
+	sigaction(SIGIO, &act, 0);
+
+	sigprocmask(SIG_UNBLOCK, &mask, 0);
+
+	debug_print("/dev/mem_notify: installed handler\n");
+}
+#endif /* linux && SIGIO */
+
 #ifdef MAEMO
 osso_context_t *get_osso_context(void)
 {
@@ -2323,11 +2381,13 @@ static void networkmanager_state_change_cb(DBusGProxy *proxy, gchar *dev,
 				went_offline_nm = FALSE;
 				main_window_toggle_work_offline(mainWin, FALSE, FALSE);
 				debug_print("NetworkManager: Went online\n");
+				log_message(LOG_PROTOCOL, _("NetworkManager: network is online.\n"));
 			}
 			else if(!online) {
 				went_offline_nm = TRUE;
 				main_window_toggle_work_offline(mainWin, TRUE, FALSE);
 				debug_print("NetworkManager: Went offline\n");
+				log_message(LOG_PROTOCOL, _("NetworkManager: network is offline.\n"));
 			}
 		}
 		else {
