@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,7 @@
 
 #include "gtk/gtkutils.h"
 #include "gtk/prefswindow.h"
+#include "combobox.h"
 
 #include "manage_window.h"
 #ifdef HAVE_LIBETPAN
@@ -88,7 +89,8 @@ static void prefs_keybind_select(void)
 
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefs_other");
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
-	gtk_window_set_title (GTK_WINDOW (window), _("Select key bindings"));
+	gtk_window_set_title (GTK_WINDOW (window), 
+				_("Choose preset keyboard shortcuts"));
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW (window), FALSE);
@@ -106,15 +108,13 @@ static void prefs_keybind_select(void)
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
-	combo = gtk_combo_new ();
-	gtk_box_pack_start (GTK_BOX (hbox1), combo, TRUE, TRUE, 0);
-	gtkut_combo_set_items (GTK_COMBO (combo),
+	combo = combobox_text_new(FALSE,
 			       _("Default"),
 			       "Mew / Wanderlust",
 			       "Mutt",
 			       _("Old Sylpheed"),
 			       NULL);
-	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO (combo)->entry), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox1), combo, TRUE, TRUE, 0);
 
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
@@ -198,8 +198,7 @@ static void prefs_keybind_apply(struct KeyBind keybind[], gint num)
 
 static void prefs_keybind_apply_clicked(GtkWidget *widget)
 {
-	GtkEntry *entry = GTK_ENTRY(GTK_COMBO(keybind.combo)->entry);
-	const gchar *text;
+	gchar *text;
 	struct KeyBind *menurc;
 	gint n_menurc;
 
@@ -447,9 +446,9 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 		{"<Compose>/Edit/Advanced/Delete a word backward",	"<control>W"},
 		{"<Compose>/Edit/Advanced/Delete a word forward",	"<alt>D"},
 	};
-  
-	text = gtk_entry_get_text(entry);
-  
+
+	text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(keybind.combo));
+
 	if (!strcmp(text, _("Default"))) {
 		menurc = default_menurc;
 		n_menurc = G_N_ELEMENTS(default_menurc);
@@ -463,8 +462,10 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 	        menurc = old_sylpheed_menurc;
 		n_menurc = G_N_ELEMENTS(old_sylpheed_menurc);
 	} else {
+		g_free(text);
 		return;
 	}
+	g_free(text);
 
 	/* prefs_keybind_apply(empty_menurc, G_N_ELEMENTS(empty_menurc)); */
 	prefs_keybind_apply(menurc, n_menurc);
@@ -535,7 +536,7 @@ static void prefs_other_create_widget(PrefsPage *_page, GtkWindow *window,
 	vbox_keys = gtkut_get_options_frame(vbox1, &frame_keys, _("Keyboard shortcuts"));
 
 	PACK_CHECK_BUTTON(vbox_keys, checkbtn_gtk_can_change_accels,
-			_("Enable customisable menu shortcuts"));
+			_("Enable customisable keyboard shortcuts"));
 	gtk_can_change_accels_tooltip = gtk_tooltips_new();
 	gtk_tooltips_set_tip(GTK_TOOLTIPS(gtk_can_change_accels_tooltip),
 			checkbtn_gtk_can_change_accels,
@@ -543,10 +544,11 @@ static void prefs_other_create_widget(PrefsPage *_page, GtkWindow *window,
 				"most of the menu items by focusing on the menu "
 				"item and pressing a key combination.\n"
 				"Uncheck this option if you want to lock all "
-				"existing menu shortcuts."),
+				"existing keyboard shortcuts."),
 			NULL);
 
-	button_keybind = gtk_button_new_with_label (_(" Set key bindings... "));
+	button_keybind = gtk_button_new_with_label(
+				_(" Choose preset keyboard shortcuts... "));
 	gtk_widget_show (button_keybind);
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox1);
