@@ -1,11 +1,11 @@
 /*
- * Sylpheed templates subsystem 
+ * Claws Mail templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
  * Copyright (C) 2001-2007 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "defs.h"
@@ -42,6 +42,7 @@
 #include "prefs_common.h"
 #include "manual.h"
 #include "gtkutils.h"
+#include "menu.h"
 
 enum {
 	TEMPL_TEXT,
@@ -56,6 +57,7 @@ static struct Templates {
 	GtkWidget *list_view;
 	GtkWidget *entry_name;
 	GtkWidget *entry_subject;
+	GtkWidget *entry_from;
 	GtkWidget *entry_to;
 	GtkWidget *entry_cc;	
 	GtkWidget *entry_bcc;
@@ -63,11 +65,30 @@ static struct Templates {
 } templates;
 
 static int modified = FALSE;
+static int modified_list = FALSE;
+
+static struct
+{
+	gchar *label;
+	GtkWidget **entry;
+	gboolean compl;
+	gchar *tooltips;
+} widgets_table[] = {
+	{N_("Name"),	&templates.entry_name,		FALSE,
+		N_("This name is used as the Menu item")},
+	{N_("From"),	&templates.entry_from,		TRUE,
+		N_("Override composing account's From header. This doesn't change the composing account.")},
+	{N_("To"),	&templates.entry_to,		TRUE, 	NULL},
+	{N_("Cc"),	&templates.entry_cc,		TRUE, 	NULL},
+	{N_("Bcc"),	&templates.entry_bcc,		TRUE, 	NULL},
+	{N_("Subject"),	&templates.entry_subject,	FALSE,	NULL},
+	{NULL,		NULL,				FALSE,	NULL}
+};
+
 
 /* widget creating functions */
 static void prefs_template_window_create	(void);
 static void prefs_template_window_setup		(void);
-static void prefs_template_clear		(void);
 
 static GSList *prefs_template_get_list		(void);
 
@@ -83,6 +104,9 @@ static void prefs_template_ok_cb		(void);
 static void prefs_template_register_cb		(void);
 static void prefs_template_substitute_cb	(void);
 static void prefs_template_delete_cb		(void);
+static void prefs_template_delete_all_cb	(void);
+static void prefs_template_clear_cb		(void);
+static void prefs_template_duplicate_cb	(void);
 static void prefs_template_top_cb		(void);
 static void prefs_template_up_cb		(void);
 static void prefs_template_down_cb	(void);
@@ -90,16 +114,12 @@ static void prefs_template_bottom_cb	(void);
 
 static GtkListStore* prefs_template_create_data_store	(void);
 static void prefs_template_list_view_insert_template	(GtkWidget *list_view,
-							 GtkTreeIter *row_iter,
+							 gint row,
 							 const gchar *template,
 							 Template *data);
 static GtkWidget *prefs_template_list_view_create	(void);
 static void prefs_template_create_list_view_columns	(GtkWidget *list_view);
-static gboolean prefs_template_selected			(GtkTreeSelection *selector,
-							 GtkTreeModel *model, 
-							 GtkTreePath *path,
-							 gboolean currently_selected,
-							 gpointer data);
+static void prefs_template_select_row(GtkTreeView *list_view, GtkTreePath *path);
 
 /* Called from mainwindow.c */
 void prefs_template_open(void)
@@ -129,9 +149,11 @@ static void prefs_template_window_create(void)
 {
 	/* window structure ;) */
 	GtkWidget *window;
+	GtkWidget *vbox;
+	GtkWidget *scrolled_window;
 	GtkWidget   *vpaned;
 	GtkWidget     *vbox1;
-	GtkWidget       *table; /* including : entry_[name|to|cc|bcc|subject] */
+	GtkWidget       *table; /* including : entry_[name|from|to|cc|bcc|subject] */
 	GtkWidget       *scroll2;
 	GtkWidget         *text_value;
 	GtkWidget     *vbox2;
@@ -141,6 +163,7 @@ static void prefs_template_window_create(void)
 	GtkWidget           *reg_btn;
 	GtkWidget           *subst_btn;
 	GtkWidget           *del_btn;
+	GtkWidget           *clear_btn;
 	GtkWidget         *desc_btn;
 	GtkWidget       *hbox4;
 	GtkWidget         *scroll1;
@@ -167,10 +190,22 @@ static void prefs_template_window_create(void)
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 
+	vbox = gtk_vbox_new(FALSE, 8);
+	gtk_widget_show(vbox);
+	gtk_container_add(GTK_CONTAINER(window), vbox);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
+
+	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show(scrolled_window);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(vbox), scrolled_window);
+
 	/* vpaned to separate template settings from templates list */
 	vpaned = gtk_vpaned_new();
 	gtk_widget_show(vpaned);
-	gtk_container_add(GTK_CONTAINER(window), vpaned);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
+ 					      vpaned);
 
 	/* vbox to handle template name and content */
 	vbox1 = gtk_vbox_new(FALSE, 6);
@@ -186,43 +221,28 @@ static void prefs_template_window_create(void)
 
 	tooltips = gtk_tooltips_new();
 
-	struct
-	{
-		gchar *label;
-		GtkWidget **entry;
-		gboolean compl;
-		gchar *tooltips;
-	} tab[] = {
-		{_("Name"),	&templates.entry_name,		FALSE,
-			_("This name is used as the Menu item")},
-		{_("To"),	&templates.entry_to,		TRUE, 	NULL},
-		{_("Cc"),	&templates.entry_cc,		TRUE, 	NULL},
-		{_("Bcc"),	&templates.entry_bcc,		TRUE, 	NULL},
-		{_("Subject"),	&templates.entry_subject,	FALSE,	NULL},
-		{NULL,		NULL,				FALSE,	NULL}
-	};
-
-	for (i=0; tab[i].label; i++) {
+	for (i=0; widgets_table[i].label; i++) {
 
 		GtkWidget *label;
 
-		label = gtk_label_new(tab[i].label);
+		label = gtk_label_new(widgets_table[i].label);
 		gtk_widget_show(label);
 		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, (i + 1),
 				(GtkAttachOptions) (GTK_FILL),
 				(GtkAttachOptions) 0, 0, 0);
 		gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
-		*(tab[i].entry) = gtk_entry_new();
-		gtk_widget_show(*(tab[i].entry));
-		gtk_table_attach(GTK_TABLE(table), *(tab[i].entry), 1, 2, i, (i + 1),
+		*(widgets_table[i].entry) = gtk_entry_new();
+		gtk_widget_show(*(widgets_table[i].entry));
+		gtk_table_attach(GTK_TABLE(table), *(widgets_table[i].entry), 1, 2, i, (i + 1),
 				(GtkAttachOptions) (GTK_EXPAND|GTK_SHRINK|GTK_FILL),
 				(GtkAttachOptions) 0, 0, 0);
-		gtk_tooltips_set_tip(tooltips, *(tab[i].entry), tab[i].tooltips, NULL);
+		gtk_tooltips_set_tip(tooltips, *(widgets_table[i].entry),
+				widgets_table[i].tooltips, NULL);
 
-		if (tab[i].compl)
+		if (widgets_table[i].compl)
 			address_completion_register_entry(
-				GTK_ENTRY(*(tab[i].entry)), TRUE);
+				GTK_ENTRY(*(widgets_table[i].entry)), TRUE);
 	}
 
 	/* template content */
@@ -247,7 +267,11 @@ static void prefs_template_window_create(void)
 		}
 	}
 	gtk_widget_show(text_value);
+#ifndef MAEMO
 	gtk_widget_set_size_request(text_value, -1, 120);
+#else
+	gtk_widget_set_size_request(text_value, -1, 60);
+#endif
 	gtk_container_add(GTK_CONTAINER(scroll2), text_value);
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(text_value), TRUE);
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_value), GTK_WRAP_WORD);
@@ -277,6 +301,8 @@ static void prefs_template_window_create(void)
 	gtk_box_pack_start(GTK_BOX(hbox3), reg_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (reg_btn), "clicked",
 			 G_CALLBACK (prefs_template_register_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), reg_btn,
+			_("Append the new template above to the list"), NULL);
 
 	subst_btn = gtkut_get_replace_btn(_("Replace"));
 	gtk_widget_show(subst_btn);
@@ -284,12 +310,24 @@ static void prefs_template_window_create(void)
 	g_signal_connect(G_OBJECT(subst_btn), "clicked",
 			 G_CALLBACK(prefs_template_substitute_cb),
 			 NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), subst_btn,
+			_("Replace the selected template in list with the template above"), NULL);
 
 	del_btn = gtk_button_new_from_stock(GTK_STOCK_DELETE);
 	gtk_widget_show(del_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), del_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(del_btn), "clicked",
 			 G_CALLBACK(prefs_template_delete_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), del_btn,
+			_("Delete the selected template from the list"), NULL);
+
+	clear_btn = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
+	gtk_widget_show (clear_btn);
+	gtk_box_pack_start (GTK_BOX (hbox3), clear_btn, FALSE, TRUE, 0);
+	g_signal_connect(G_OBJECT (clear_btn), "clicked",
+			G_CALLBACK(prefs_template_clear_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), clear_btn,
+			_("Clear all the input fields in the dialog"), NULL);
 
 #if GTK_CHECK_VERSION(2, 8, 0)
 	desc_btn = gtk_button_new_from_stock(GTK_STOCK_INFO);
@@ -299,7 +337,9 @@ static void prefs_template_window_create(void)
 	gtk_widget_show(desc_btn);
 	gtk_box_pack_end(GTK_BOX(hbox2), desc_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(desc_btn), "clicked",
-			 G_CALLBACK(quote_fmt_quote_description), NULL);
+			 G_CALLBACK(quote_fmt_quote_description), window);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), desc_btn,
+			_("Show information on configuring templates"), NULL);
 
 	/* templates list */
 	hbox4 = gtk_hbox_new(FALSE, 8);
@@ -322,6 +362,8 @@ static void prefs_template_window_create(void)
 	gtk_box_pack_start(GTK_BOX(vbox3), top_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(top_btn), "clicked",
 			 G_CALLBACK(prefs_template_top_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), top_btn,
+			_("Move the selected template to the top"), NULL);
 
 	PACK_VSPACER(vbox3, spc_vbox, VSPACING_NARROW_2);
 
@@ -330,12 +372,16 @@ static void prefs_template_window_create(void)
 	gtk_box_pack_start (GTK_BOX(vbox3), up_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(up_btn), "clicked",
 			 G_CALLBACK(prefs_template_up_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), up_btn,
+			_("Move the selected template up"), NULL);
 
 	down_btn = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
 	gtk_widget_show (down_btn);
 	gtk_box_pack_start(GTK_BOX (vbox3), down_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (down_btn), "clicked",
 			 G_CALLBACK(prefs_template_down_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), down_btn,
+			_("Move the selected template down"), NULL);
 
 	PACK_VSPACER(vbox3, spc_vbox, VSPACING_NARROW_2);
 
@@ -344,6 +390,8 @@ static void prefs_template_window_create(void)
 	gtk_box_pack_start(GTK_BOX(vbox3), bottom_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(bottom_btn), "clicked",
 			 G_CALLBACK(prefs_template_bottom_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), bottom_btn,
+			_("Move the selected template to the bottom"), NULL);
 
 	list_view = prefs_template_list_view_create();
 	gtk_widget_show(list_view);
@@ -356,7 +404,7 @@ static void prefs_template_window_create(void)
 			&ok_btn, GTK_STOCK_OK,
 			NULL, NULL);
 	gtk_widget_show(confirm_area);
-	gtk_box_pack_end(GTK_BOX(vbox2), confirm_area, FALSE, FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
 	gtk_window_set_title(GTK_WINDOW(window), _("Template configuration"));
@@ -379,7 +427,7 @@ static void prefs_template_window_create(void)
 	address_completion_start(window);
 
 	if (!geometry.min_height) {
-		geometry.min_width = 480;
+		geometry.min_width = 500;
 		geometry.min_height = 540;
 	}
 
@@ -394,36 +442,12 @@ static void prefs_template_window_create(void)
 	templates.text_value = text_value;
 }
 
-static void prefs_template_window_setup(void)
+static void prefs_template_reset_dialog(void)
 {
-	GSList *tmpl_list;
-	GSList *cur;
-	Template *tmpl;
-	GtkListStore *store;
 	GtkTextBuffer *buffer;
-	
-	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
-				(templates.list_view)));
-
-	manage_window_set_transient(GTK_WINDOW(templates.window));
-	gtk_widget_grab_focus(templates.ok_btn);
-
-	gtk_list_store_clear(store);
-
-	prefs_template_list_view_insert_template(templates.list_view,
-						 NULL, _("(New)"),
-						 NULL);
-	
-	tmpl_list = template_read_config();
-
-	for (cur = tmpl_list; cur != NULL; cur = cur->next) {
-		tmpl = (Template *)cur->data;
-		prefs_template_list_view_insert_template(templates.list_view,
-							 NULL, tmpl->name, 
-							 tmpl);
-	}
 
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_from), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_to), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc), "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_bcc), "");			
@@ -431,17 +455,44 @@ static void prefs_template_window_setup(void)
 	
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(templates.text_value));
 	gtk_text_buffer_set_text(buffer, "", -1);
-
-	g_slist_free(tmpl_list);
 }
 
-static void prefs_template_clear(void)
+static void prefs_template_clear_list(void)
 {
 	GtkListStore *store;
-
+	
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
 				(templates.list_view)));
 	gtk_list_store_clear(store);
+
+	prefs_template_list_view_insert_template(templates.list_view,
+						 -1, _("(New)"),
+						 NULL);
+}
+
+static void prefs_template_window_setup(void)
+{
+	GSList *tmpl_list;
+	GSList *cur;
+	Template *tmpl;
+
+	manage_window_set_transient(GTK_WINDOW(templates.window));
+	gtk_widget_grab_focus(templates.ok_btn);
+
+	prefs_template_clear_list();
+	
+	tmpl_list = template_read_config();
+
+	for (cur = tmpl_list; cur != NULL; cur = cur->next) {
+		tmpl = (Template *)cur->data;
+		prefs_template_list_view_insert_template(templates.list_view,
+							 -1, tmpl->name, 
+							 tmpl);
+	}
+
+	prefs_template_reset_dialog();
+
+	g_slist_free(tmpl_list);
 }
 
 static gint prefs_template_deleted_cb(GtkWidget *widget, GdkEventAny *event,
@@ -466,35 +517,67 @@ static gboolean prefs_template_key_pressed_cb(GtkWidget *widget,
 	return FALSE;
 }
 
+static void prefs_template_address_completion_end(void)
+{
+	gint i;
+
+	for (i=0; widgets_table[i].label; i++) {
+		if (widgets_table[i].compl)
+			address_completion_unregister_entry(
+				GTK_ENTRY(*(widgets_table[i].entry)));
+	}
+	address_completion_end(templates.window);
+}
+
 static void prefs_template_ok_cb(void)
 {
 	GSList *tmpl_list;
+	GtkListStore *store;
 
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
 				 GTK_STOCK_CLOSE, _("+_Continue editing"), 
 				 NULL) != G_ALERTDEFAULT) {
 		return;
-	}
+	} 
+
+	prefs_template_address_completion_end();
+
 	modified = FALSE;
+	modified_list = FALSE;
 	tmpl_list = prefs_template_get_list();
 	template_set_config(tmpl_list);
 	compose_reflect_prefs_all();
-	prefs_template_clear();
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
+				(templates.list_view)));
+	gtk_list_store_clear(store);
 	gtk_widget_hide(templates.window);
 	inc_unlock();
 }
 
 static void prefs_template_cancel_cb(void)
 {
+	GtkListStore *store;
+
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
 				 GTK_STOCK_CLOSE, _("+_Continue editing"),
 				 NULL) != G_ALERTDEFAULT) {
 		return;
+	} else if (modified_list && alertpanel(_("Templates list not saved"),
+				 _("The templates list has been modified. Close anyway?"),
+				 GTK_STOCK_CLOSE, _("+_Continue editing"), 
+				 NULL) != G_ALERTDEFAULT) {
+		return;
 	}
+
+	prefs_template_address_completion_end();
+
 	modified = FALSE;
-	prefs_template_clear();
+	modified_list = FALSE;
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
+				(templates.list_view)));
+	gtk_list_store_clear(store);
 	gtk_widget_hide(templates.window);
 	inc_unlock();
 }
@@ -530,6 +613,9 @@ static GSList *prefs_template_get_list(void)
 			ntmpl->subject = tmpl->subject && *(tmpl->subject) 
 					 ? g_strdup(tmpl->subject) 
 					 : NULL;
+			ntmpl->from    = tmpl->from && *(tmpl->from)
+					 ? g_strdup(tmpl->from)
+					 : NULL;
 			ntmpl->to      = tmpl->to && *(tmpl->to)
 					 ? g_strdup(tmpl->to)
 					 : NULL;
@@ -550,17 +636,19 @@ static GSList *prefs_template_get_list(void)
 	return tmpl_list;
 }
 
-gboolean prefs_template_string_is_valid(gchar *string, gint *line)
+gboolean prefs_template_string_is_valid(gchar *string, gint *line, gboolean escaped_string, gboolean email)
 {
+	gboolean result = TRUE;
 	if (string && *string != '\0') {
 		gchar *parsed_buf;
 		MsgInfo dummyinfo;
+		PrefsAccount *account = account_get_default();
 
 		memset(&dummyinfo, 0, sizeof(MsgInfo));
 #ifdef USE_ASPELL
-		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, NULL, NULL);
+		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, account, escaped_string, NULL);
 #else
-		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, NULL);
+		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, account, escaped_string);
 #endif
 		quote_fmt_scan_string(string);
 		quote_fmt_parse();
@@ -570,17 +658,28 @@ gboolean prefs_template_string_is_valid(gchar *string, gint *line)
 				*line = quote_fmt_get_line();
 			return FALSE;
 		}
+		if (email) {
+			const gchar *start = strrchr(parsed_buf, '<');
+			const gchar *end = strrchr(parsed_buf, '>');
+			const gchar *at = strrchr(parsed_buf, '@');
+			const gchar *space = strrchr(parsed_buf, ' ');
+			if (!at)
+				result = FALSE;
+			if (at && space && (!start || !end || end < start || start < space))
+				result = FALSE;
+		}
 		quote_fmt_reset_vartable();
 	}
-	return TRUE;
+	return result;
 }
 
-static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
+static gboolean prefs_template_list_view_set_row(gint row)
 /* return TRUE if the row could be modified */
 {
 	Template *tmpl;
 	gchar *name;
 	gchar *subject;
+	gchar *from;
 	gchar *to;
 	gchar *cc;
 	gchar *bcc;	
@@ -601,12 +700,8 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 			g_free(value);
 		value = NULL;
 		}
-	if (!prefs_template_string_is_valid(value, &line)) {
-		gchar *msg;
-
-		msg = g_strdup_printf(_("Template body format error at line %d."), line);
-		alertpanel_error(msg);
-		g_free(msg);
+	if (!prefs_template_string_is_valid(value, &line, FALSE, FALSE)) {
+		alertpanel_error(_("Template body format error at line %d."), line);
 		g_free(value);
 		return FALSE;
 	}
@@ -615,8 +710,11 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 				      0, -1);
 	if (*name == '\0') {
 		alertpanel_error(_("Template name is not set."));
+		g_free(value);
 		return FALSE;
 	}
+	from = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_from),
+				    0, -1);
 	to = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_to),
 				    0, -1);
 	cc = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_cc),
@@ -626,6 +724,10 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 	subject = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_subject),
 					 0, -1);
 
+	if (from && *from == '\0') {
+		g_free(from);
+		from = NULL;
+	}
 	if (to && *to == '\0') {
 		g_free(to);
 		to = NULL;
@@ -643,24 +745,34 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 		subject = NULL;
 	}
 
-	if (!prefs_template_string_is_valid(to, NULL)) {
+	if (!prefs_template_string_is_valid(from, NULL, FALSE, TRUE)) {
+		alertpanel_error(_("Template From format error."));
+		g_free(from);
+		g_free(value);
+		return FALSE;
+	}
+	if (!prefs_template_string_is_valid(to, NULL, FALSE, TRUE)) {
 		alertpanel_error(_("Template To format error."));
 		g_free(to);
+		g_free(value);
 		return FALSE;
 	}
-	if (!prefs_template_string_is_valid(cc, NULL)) {
+	if (!prefs_template_string_is_valid(cc, NULL, FALSE, TRUE)) {
 		alertpanel_error(_("Template Cc format error."));	
 		g_free(cc);
+		g_free(value);
 		return FALSE;
 	}
-	if (!prefs_template_string_is_valid(bcc, NULL)) {
+	if (!prefs_template_string_is_valid(bcc, NULL, FALSE, TRUE)) {
 		alertpanel_error(_("Template Bcc format error."));	
 		g_free(bcc);
+		g_free(value);
 		return FALSE;
 	}
-	if (!prefs_template_string_is_valid(subject, NULL)) {
+	if (!prefs_template_string_is_valid(subject, NULL, FALSE, FALSE)) {
 		alertpanel_error(_("Template subject format error."));	
 		g_free(subject);
+		g_free(value);
 		return FALSE;
 	}
 	
@@ -668,6 +780,7 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 	tmpl->load_filename = NULL;
 	tmpl->name = name;
 	tmpl->subject = subject;
+	tmpl->from = from;
 	tmpl->to = to;
 	tmpl->cc = cc;
 	tmpl->bcc = bcc;	
@@ -675,54 +788,56 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 
 	prefs_template_list_view_insert_template(templates.list_view,
 						 row, tmpl->name, tmpl);
+
 	return TRUE;
 }
 
 static void prefs_template_register_cb(void)
 {
-	modified = !prefs_template_list_view_set_row(NULL);
+	modified = !prefs_template_list_view_set_row(-1);
+	modified_list = TRUE;
 }
 
 static void prefs_template_substitute_cb(void)
 {
 	Template *tmpl;
-	GtkTreeIter row;
-	GtkTreeSelection *selection;
+	gint row;
+	GtkTreeIter iter;
 	GtkTreeModel *model;
 
-	selection = gtk_tree_view_get_selection
-			(GTK_TREE_VIEW(templates.list_view));
-	
-	if (!gtk_tree_selection_get_selected(selection, &model, &row))
+	row = gtkut_list_view_get_selected_row(templates.list_view);
+	if (row <= 0)
 		return;
 
-	gtk_tree_model_get(model, &row, 
-			   TEMPL_DATA, &tmpl,
-			   -1);
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view));
+	if (!gtk_tree_model_iter_nth_child(model, &iter, NULL, row))
+		return;
 
-	if (!tmpl) return;
+	gtk_tree_model_get(model, &iter, TEMPL_DATA, &tmpl, -1);
+	if (!tmpl)
+		return;
 
-	modified = !prefs_template_list_view_set_row(&row);
+	modified = !prefs_template_list_view_set_row(row);
+	modified_list = TRUE;
 }
 
 static void prefs_template_delete_cb(void)
 {
 	Template *tmpl;
-	GtkTreeIter row;
-	GtkTreeSelection *selection;
+	gint row;
+	GtkTreeIter iter;
 	GtkTreeModel *model;
 
-	selection = gtk_tree_view_get_selection
-			(GTK_TREE_VIEW(templates.list_view));
-	
-	if (!gtk_tree_selection_get_selected(selection, &model, &row))
+	row = gtkut_list_view_get_selected_row(templates.list_view);
+	if (row <= 0)
 		return;
 
-	gtk_tree_model_get(model, &row, 
-			   TEMPL_DATA, &tmpl,
-			   -1);
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view));
+	if (!gtk_tree_model_iter_nth_child(model, &iter, NULL, row))
+		return;
 
-	if (!tmpl) 
+	gtk_tree_model_get(model, &iter, TEMPL_DATA, &tmpl, -1);
+	if (!tmpl)
 		return;
 
 	if (alertpanel(_("Delete template"),
@@ -731,7 +846,52 @@ static void prefs_template_delete_cb(void)
 		       NULL) != G_ALERTALTERNATE)
 		return;
 
-	gtk_list_store_remove(GTK_LIST_STORE(model), &row);		
+	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+	modified_list = TRUE;	
+}
+
+static void prefs_template_delete_all_cb(void)
+{
+	GtkListStore *list_store;
+
+	if (alertpanel(_("Delete all templates"),
+			  _("Do you really want to delete all the templates?"),
+			  GTK_STOCK_CANCEL, "+"GTK_STOCK_DELETE, NULL) == G_ALERTDEFAULT)
+	   return;
+
+	list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view)));
+	prefs_template_clear_list();
+	modified = FALSE;
+
+	prefs_template_reset_dialog();
+	modified_list = TRUE;
+}
+
+static void prefs_template_duplicate_cb(void)
+{
+	Template *tmpl;
+	gint row;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	
+	row = gtkut_list_view_get_selected_row(templates.list_view);
+	if (row <= 0)
+		return;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view));
+	if (!gtk_tree_model_iter_nth_child(model, &iter, NULL, row))
+		return;
+
+	gtk_tree_model_get(model, &iter, TEMPL_DATA, &tmpl, -1);
+	if (!tmpl)
+		return;
+
+	modified_list = !prefs_template_list_view_set_row(-row-2);
+}
+
+static void prefs_template_clear_cb(void)
+{
+   prefs_template_reset_dialog();
 }
 
 static void prefs_template_top_cb(void)
@@ -752,6 +912,7 @@ static void prefs_template_top_cb(void)
 
 	gtk_list_store_move_after(GTK_LIST_STORE(model), &sel, &top);
 	gtkut_list_view_select_row(templates.list_view, 1);
+	modified_list = TRUE;
 }
 
 static void prefs_template_up_cb(void)
@@ -772,6 +933,7 @@ static void prefs_template_up_cb(void)
 
 	gtk_list_store_swap(GTK_LIST_STORE(model), &top, &sel);
 	gtkut_list_view_select_row(templates.list_view, row - 1);
+	modified_list = TRUE;
 }
 
 static void prefs_template_down_cb(void)
@@ -792,6 +954,7 @@ static void prefs_template_down_cb(void)
 			
 	gtk_list_store_swap(GTK_LIST_STORE(model), &top, &sel);
 	gtkut_list_view_select_row(templates.list_view, row + 1);
+	modified_list = TRUE;
 }
 
 static void prefs_template_bottom_cb(void)
@@ -812,6 +975,7 @@ static void prefs_template_bottom_cb(void)
 
 	gtk_list_store_move_after(GTK_LIST_STORE(model), &top, &sel);		
 	gtkut_list_view_select_row(templates.list_view, n_rows - 1);
+	modified_list = TRUE;
 }
 
 static GtkListStore* prefs_template_create_data_store(void)
@@ -824,35 +988,132 @@ static GtkListStore* prefs_template_create_data_store(void)
 }
 
 static void prefs_template_list_view_insert_template(GtkWidget *list_view,
-						     GtkTreeIter *row_iter,
+						     gint row,
 						     const gchar *template,
 						     Template *data)
 {
 	GtkTreeIter iter;
-	GtkTreeModel *model = gtk_tree_view_get_model
-					(GTK_TREE_VIEW(list_view));
-	GtkListStore *list_store = GTK_LIST_STORE(model);
-	GAuto *auto_data;					
+	GtkTreeIter sibling;
+	GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model
+					(GTK_TREE_VIEW(list_view)));
 
-	if (row_iter == NULL) {
+/*	row -1 to add a new rule to store,
+	row >=0 to change an existing row
+	row <-1 insert a new row after (-row-2)
+*/
+	if (row >= 0 ) {
+		/* modify the existing */
+ 		if (!gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(list_store),
+ 						   &iter, NULL, row))
+			row = -1;
+	} else if (row < -1 ) {
+		if (!gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(list_store),
+						   &sibling, NULL, -row-2))
+ 			row = -1;		
+	}
+
+	if (row == -1 ) {
 		/* append new */
 		gtk_list_store_append(list_store, &iter);
+		gtk_list_store_set(list_store, &iter,
+				   TEMPL_TEXT, template,
+				   TEMPL_DATA, data,
+				   -1);
+	} else if (row < -1) {
+		/* duplicate */
+		gtk_list_store_insert_after(list_store, &iter, &sibling);
+		gtk_list_store_set(list_store, &iter,
+				   TEMPL_TEXT, template,
+				   TEMPL_DATA, data,
+				   -1);
 	} else {
-		/* modify the existing */
-		iter = *row_iter;
+		/* change existing */
+		GAuto *auto_data =
+					g_auto_pointer_new_with_free(data, (GFreeFunc) template_free);  
+
+		/* if replacing data in an existing row, the auto pointer takes care
+		 * of destroying the Template data */
+		gtk_list_store_set(list_store, &iter,
+				   TEMPL_TEXT, template,
+				   TEMPL_DATA, data,
+				   TEMPL_AUTO_DATA, auto_data,
+				   -1);
+
+		g_auto_pointer_free(auto_data);
 	}
-		
-	auto_data = g_auto_pointer_new_with_free(data, (GFreeFunc) template_free);  
+}
 
-	/* if replacing data in an existing row, the auto pointer takes care
-	 * of destroying the Template data */
-	gtk_list_store_set(list_store, &iter,
-			   TEMPL_TEXT, template,
-			   TEMPL_DATA, data,
-			   TEMPL_AUTO_DATA, auto_data,
-			   -1);
+static GtkItemFactory *prefs_template_popup_factory = NULL;
+static GtkWidget *prefs_template_popup_menu = NULL;
 
-	g_auto_pointer_free(auto_data);			   
+static GtkItemFactoryEntry prefs_template_popup_entries[] =
+{
+   {N_("/_Delete"),		NULL, prefs_template_delete_cb, 0, NULL, NULL},
+   {N_("/Delete _all"),	NULL, prefs_template_delete_all_cb, 0, NULL, NULL},
+   {N_("/D_uplicate"),	NULL, prefs_template_duplicate_cb, 0, NULL, NULL},
+};
+
+static gint prefs_template_list_btn_pressed(GtkWidget *widget, GdkEventButton *event,
+				   GtkTreeView *list_view)
+{
+   if (event) {
+	   /* left- or right-button click */
+	   if (event->button == 1 || event->button == 3) {
+		   GtkTreePath *path = NULL;
+		   if (gtk_tree_view_get_path_at_pos( list_view, event->x, event->y,
+							   &path, NULL, NULL, NULL)) {
+			   prefs_template_select_row(list_view, path);
+		   }
+		   if (path)
+			   gtk_tree_path_free(path);
+	   }
+
+	   /* right-button click */
+	   if (event->button == 3) {
+		   GtkTreeModel *model = gtk_tree_view_get_model(list_view);
+		   GtkTreeIter iter;
+		   gboolean non_empty;
+		   gint row;
+
+		   if (!prefs_template_popup_menu) {
+			   gint n_entries = sizeof(prefs_template_popup_entries) /
+					   sizeof(prefs_template_popup_entries[0]);
+			   prefs_template_popup_menu = menu_create_items(prefs_template_popup_entries,
+								 n_entries, "<PrefsTemplatePopupMenu>",
+								 &prefs_template_popup_factory, list_view);
+		   }
+
+		   /* grey out some popup menu items if there is no selected row */
+		   row = gtkut_list_view_get_selected_row(GTK_WIDGET(list_view));
+		   menu_set_sensitive(prefs_template_popup_factory, "/Delete", (row > 0));
+		   menu_set_sensitive(prefs_template_popup_factory, "/Duplicate", (row > 0));
+
+		   /* grey out seom popup menu items if there is no row
+			  (not counting the (New) one at row 0) */
+		   non_empty = gtk_tree_model_get_iter_first(model, &iter);
+		   if (non_empty)
+			   non_empty = gtk_tree_model_iter_next(model, &iter);
+		   menu_set_sensitive(prefs_template_popup_factory, "/Delete all", non_empty);
+
+		   gtk_menu_popup(GTK_MENU(prefs_template_popup_menu), 
+					  NULL, NULL, NULL, NULL, 
+					  event->button, event->time);
+	   }
+   }
+   return FALSE;
+}
+
+static gboolean prefs_template_list_popup_menu(GtkWidget *widget, gpointer data)
+{
+   GtkTreeView *list_view = (GtkTreeView *)data;
+   GdkEventButton event;
+   
+   event.button = 3;
+   event.time = gtk_get_current_event_time();
+   
+   prefs_template_list_btn_pressed(NULL, &event, list_view);
+
+   return TRUE;
 }
 
 static GtkWidget *prefs_template_list_view_create(void)
@@ -864,14 +1125,24 @@ static GtkWidget *prefs_template_list_view_create(void)
 	model = GTK_TREE_MODEL(prefs_template_create_data_store());
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	g_object_unref(model);	
+
+#ifndef MAEMO
+	g_signal_connect(G_OBJECT(list_view), "popup-menu",
+			 G_CALLBACK(prefs_template_list_popup_menu), list_view);
+#else
+	gtk_widget_tap_and_hold_setup(GTK_WIDGET(list_view), NULL, NULL,
+			GTK_TAP_AND_HOLD_NONE | GTK_TAP_AND_HOLD_NO_INTERNALS);
+	g_signal_connect(G_OBJECT(list_view), "tap-and-hold",
+			 G_CALLBACK(prefs_template_list_popup_menu), list_view);
+#endif
+	g_signal_connect(G_OBJECT(list_view), "button-press-event",
+			G_CALLBACK(prefs_template_list_btn_pressed), list_view);
 	
 	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
 	gtk_tree_view_set_reorderable(list_view, TRUE);
 
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
-	gtk_tree_selection_set_select_function(selector, prefs_template_selected,
-					       NULL, NULL);
 
 	/* create the columns */
 	prefs_template_create_list_view_columns(GTK_WIDGET(list_view));
@@ -893,39 +1164,41 @@ static void prefs_template_create_list_view_columns(GtkWidget *list_view)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);		
 }
 
-static gboolean prefs_template_selected(GtkTreeSelection *selector,
-				        GtkTreeModel *model, 
-				        GtkTreePath *path,
-				        gboolean currently_selected,
-				        gpointer data)
+ /*!
+ *\brief	Triggered when a row has to be selected
+ */
+static void prefs_template_select_row(GtkTreeView *list_view, GtkTreePath *path)
 {
+	GtkTreeModel *model = gtk_tree_view_get_model(list_view);
+	GtkTreeSelection *selection;
 	Template *tmpl;
 	Template tmpl_def;
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
 	GtkTreeIter titer;
 
-	if (currently_selected)
-		return TRUE;
+	if (!model || !path || !gtk_tree_model_get_iter(model, &titer, path))
+		return;
 
-	if (!gtk_tree_model_get_iter(model, &titer, path))
-		return TRUE;
+	/* select row */
+	selection = gtk_tree_view_get_selection(list_view);
+	gtk_tree_selection_select_path(selection, path);
 
 	tmpl_def.name = _("Template");
 	tmpl_def.subject = "";
+	tmpl_def.from = "";
 	tmpl_def.to = "";
 	tmpl_def.cc = "";
 	tmpl_def.bcc = "";	
 	tmpl_def.value = "";
 
-	gtk_tree_model_get(model, &titer,
-			   TEMPL_DATA, &tmpl,
-			   -1);
-
-	if (!tmpl) 
+	gtk_tree_model_get(model, &titer, TEMPL_DATA, &tmpl, -1);
+	if (!tmpl)
 		tmpl = &tmpl_def;
 
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), tmpl->name);
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_from),
+			   tmpl->from ? tmpl->from : "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_to),
 			   tmpl->to ? tmpl->to : "");
 	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc),
@@ -939,6 +1212,4 @@ static gboolean prefs_template_selected(GtkTreeSelection *selector,
 	gtk_text_buffer_set_text(buffer, "", -1);
 	gtk_text_buffer_get_start_iter(buffer, &iter);
 	gtk_text_buffer_insert(buffer, &iter, tmpl->value, -1);
-
-	return TRUE;
 }
