@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -62,10 +62,11 @@ static gchar *quote_desc_strings[] = {
 	"%quoted_msg (%Q)",			N_("quoted message body"), /* quoted message */
 	"%msg_no_sig (%m)",			N_("message body without signature"), /* message with no signature */
 	"%quoted_msg_no_sig (%q)",	N_("quoted message body without signature"), /* quoted message with no signature */
+	"%tags",				N_("message tags"), /* message tags */
 	"%dict (%T)",				N_("current dictionary"), /* current dictionary */
 	"%cursor (%X)",				N_("cursor position"), /* X marks the cursor spot */
 	"%account_fullname (%af)",	N_("account property: your name"), /* full name in compose account */
-	"%account_email (%ae)",		N_("account property: your email address"), /* mail address in compose account */
+	"%account_email (%am)",		N_("account property: your email address"), /* mail address in compose account */
 	"%account_name (%an)",		N_("account property: account name"), /* compose account name itself */
 	"%account_org (%ao)",		N_("account property: organization"), /* organization in compose account */
 	"%account_dict (%aT)",		N_("account property: default dictionary"), /* main dict (if enabled) in account */
@@ -88,6 +89,7 @@ static gchar *quote_desc_strings[] = {
 	"|file{<span style=\"oblique\">sub_expr</span>}\n(|f{<span style=\"oblique\">sub_expr</span>})",		N_("insert file:\n<span style=\"oblique\">sub_expr</span> is evaluated as the path of the file to insert"), /* insert file */
 	"|program{<span style=\"oblique\">sub_expr</span>}\n(|p{<span style=\"oblique\">sub_expr</span>})\n",	N_("insert program output:\n<span style=\"oblique\">sub_expr</span> is evaluated as a command-line to get\nthe output from"), /* insert program output */
 	"|input{<span style=\"oblique\">sub_expr</span>}\n(|i{<span style=\"oblique\">sub_expr</span>})\n",		N_("insert user input:\n<span style=\"oblique\">sub_expr</span> is a variable to be replaced by\nuser-entered text"), /* insert user input */
+	"|attach{<span style=\"oblique\">sub_expr</span>}\n(|a{<span style=\"oblique\">sub_expr</span>})",		N_("attach file:\n<span style=\"oblique\">sub_expr</span> is evaluated as the path of the file to attach"), /* attach file */
 	"",					NULL,
 	N_("<span weight=\"bold\">definition of terms:</span>"),	NULL,
 	"<span style=\"oblique\">expr</span>\n",			N_("text that can contain any of the symbols or\ncommands above"),
@@ -115,15 +117,18 @@ void quote_fmt_quote_description(GtkWidget *widget, GtkWidget *pref_window)
 void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 						GtkWidget *parent_box,
 						GtkWidget **checkbtn_compose_with_format,
-						gchar *checkbtn_compose_text,
+						GtkWidget **override_from_format,
 						GtkWidget **edit_subject_format,
 						GtkWidget **edit_body_format,
-						gboolean add_info_button)
+						gboolean add_info_button,
+						void(*set_defaults_func)(void))
 {
 	GtkWidget *checkbtn_use_format = NULL;
-	GtkWidget *frame_format;
 	GtkWidget *vbox_format;
 	GtkWidget *hbox_format;
+	GtkWidget *hbox2_format;
+	GtkWidget *label_from;
+	GtkWidget *entry_from = NULL;
 	GtkWidget *label_subject;
 	GtkWidget *entry_subject;
 	GtkWidget *scrolledwin_format;
@@ -132,21 +137,39 @@ void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 	if (add_info_button)
 		g_return_if_fail(parent_window != NULL);
 	g_return_if_fail(parent_box != NULL);
-	if (checkbtn_compose_with_format) {
+	if (checkbtn_compose_with_format)
 		g_return_if_fail(checkbtn_compose_with_format != NULL);
-		g_return_if_fail(checkbtn_compose_text != NULL);
-	}
+
 	g_return_if_fail(edit_subject_format != NULL);
 	g_return_if_fail(edit_body_format != NULL);
 
 	if (checkbtn_compose_with_format)
 		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format, 
-				   _("Use format when composing new messages"));
+				   _("Use template when composing new messages"));
 
-	vbox_format = gtkut_get_options_frame(parent_box, &frame_format, checkbtn_compose_text);
+	vbox_format = gtk_vbox_new (FALSE, 4);
+	gtk_widget_show(vbox_format);
+	gtk_container_add(GTK_CONTAINER (parent_box), vbox_format);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox_format), 8);
 
-	if (checkbtn_compose_with_format)
-		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, frame_format);
+	if (override_from_format) {
+		hbox2_format = gtk_hbox_new (FALSE, 8);
+		gtk_widget_show (hbox2_format);
+		gtk_box_pack_start (GTK_BOX (vbox_format), hbox2_format, FALSE, FALSE, 0);
+
+		label_from = gtk_label_new (_("From"));
+		gtk_widget_show (label_from);
+		gtk_box_pack_start (GTK_BOX (hbox2_format), label_from, FALSE, FALSE, 0);
+
+		entry_from = gtk_entry_new ();
+		gtk_widget_show (entry_from);
+		gtk_box_pack_start (GTK_BOX (hbox2_format), entry_from, TRUE, TRUE, 0);
+		gtk_widget_set_size_request (entry_from, 100, -1);
+
+		gtk_tooltips_set_tip(gtk_tooltips_new(), entry_from,
+				_("Override From header. This doesn't change the account used to compose the new message."),
+				NULL);
+	}
 
 	hbox_format = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox_format);
@@ -187,11 +210,23 @@ void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (text_format), TRUE);
 	gtk_widget_set_size_request(text_format, -1, 100);
 
+	if (checkbtn_compose_with_format) {
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_subject);
+		if (override_from_format)
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_from);
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_subject);
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, text_format);
+	}
+
 	if (add_info_button)
 		quotefmt_add_info_button(parent_window, vbox_format);
+	if (set_defaults_func)
+		quotefmt_add_defaults_button(parent_window, vbox_format, set_defaults_func);
 
 	if (checkbtn_compose_with_format)
 		*checkbtn_compose_with_format = checkbtn_use_format;
+	if (override_from_format)
+		*override_from_format = entry_from;
 	*edit_subject_format = entry_subject;
 	*edit_body_format = text_format;
 }
@@ -199,44 +234,61 @@ void quotefmt_create_new_msg_fmt_widgets(GtkWindow *parent_window,
 void quotefmt_create_reply_fmt_widgets(GtkWindow *parent_window,
 						GtkWidget *parent_box,
 						GtkWidget **checkbtn_reply_with_format,
-						gchar *checkbtn_reply_text,
+						GtkWidget **override_from_format,
 						GtkWidget **edit_reply_quotemark,
 						GtkWidget **edit_reply_format,
-						gboolean add_info_button)
+						gboolean add_info_button,
+						void(*set_defaults_func)(void))
 {
 	GtkWidget *checkbtn_use_format = NULL;
-	GtkWidget *frame_quote;
 	GtkWidget *vbox_quote;
 	GtkWidget *hbox1;
 	GtkWidget *hbox2;
+	GtkWidget *hbox3;
 	GtkWidget *label_quotemark;
 	GtkWidget *entry_quotemark;
+	GtkWidget *label_from;
+	GtkWidget *entry_from = NULL;
 	GtkWidget *scrolledwin_quotefmt;
 	GtkWidget *text_quotefmt;
 
 	if (add_info_button)
 		g_return_if_fail(parent_window != NULL);
 	g_return_if_fail(parent_box != NULL);
-	if (checkbtn_reply_with_format) {
+	if (checkbtn_reply_with_format)
 		g_return_if_fail(checkbtn_reply_with_format != NULL);
-		g_return_if_fail(checkbtn_reply_text != NULL);
-	}
+
 	g_return_if_fail(edit_reply_quotemark != NULL);
 	g_return_if_fail(edit_reply_format != NULL);
 
 	if (checkbtn_reply_with_format)
-		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format, checkbtn_reply_text);
+		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format,
+				   _("Use template when replying to messages"));
 
-	PACK_FRAME (parent_box, frame_quote, _("Reply format"));
-
-	if (checkbtn_reply_with_format)
-		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, frame_quote);
-
-	vbox_quote = gtk_vbox_new (FALSE, VSPACING_NARROW);
-	gtk_widget_show (vbox_quote);
-	gtk_container_add (GTK_CONTAINER (frame_quote), vbox_quote);
+	vbox_quote = gtk_vbox_new (FALSE, 4);
+	gtk_widget_show(vbox_quote);
+	gtk_container_add(GTK_CONTAINER (parent_box), vbox_quote);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);
 
+	if (override_from_format) {
+		hbox3 = gtk_hbox_new (FALSE, 8);
+		gtk_widget_show (hbox3);
+		gtk_box_pack_start (GTK_BOX (vbox_quote), hbox3, FALSE, FALSE, 0);
+
+		label_from = gtk_label_new (_("From"));
+		gtk_widget_show (label_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), label_from, FALSE, FALSE, 0);
+
+		entry_from = gtk_entry_new ();
+		gtk_widget_show (entry_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), entry_from, TRUE, TRUE, 0);
+		gtk_widget_set_size_request (entry_from, 100, -1);
+
+		gtk_tooltips_set_tip(gtk_tooltips_new(), entry_from,
+				_("Override From header. This doesn't change the account used to reply."),
+				NULL);
+	}
+
 	hbox1 = gtk_hbox_new (FALSE, 32);
 	gtk_widget_show (hbox1);
 	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, FALSE, FALSE, 0);
@@ -280,30 +332,45 @@ void quotefmt_create_reply_fmt_widgets(GtkWindow *parent_window,
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (text_quotefmt), TRUE);
 	gtk_widget_set_size_request(text_quotefmt, -1, 100);
 
+	if (checkbtn_reply_with_format) {
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_quotemark);
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_quotemark);
+		if (override_from_format)
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_from);
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, text_quotefmt);
+	}
+
 	if (add_info_button)
 		quotefmt_add_info_button(parent_window, vbox_quote);
+	if (set_defaults_func)
+		quotefmt_add_defaults_button(parent_window, vbox_quote, set_defaults_func);
 
 	if (checkbtn_reply_with_format)
 		*checkbtn_reply_with_format = checkbtn_use_format;
 	*edit_reply_quotemark = entry_quotemark;
+	if (override_from_format)
+		*override_from_format = entry_from;
 	*edit_reply_format = text_quotefmt;
 }
 
 void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 						GtkWidget *parent_box,
 						GtkWidget **checkbtn_forward_with_format,
-						gchar *checkbtn_forward_text,
+						GtkWidget **override_from_format,
 						GtkWidget **edit_fw_quotemark,
 						GtkWidget **edit_fw_format,
-						gboolean add_info_button)
+						gboolean add_info_button,
+						void(*set_defaults_func)(void))
 {
 	GtkWidget *checkbtn_use_format = NULL;
-	GtkWidget *frame_quote;
 	GtkWidget *vbox_quote;
 	GtkWidget *hbox1;
 	GtkWidget *hbox2;
+	GtkWidget *hbox3;
 	GtkWidget *label_quotemark;
 	GtkWidget *entry_fw_quotemark;
+	GtkWidget *label_from;
+	GtkWidget *entry_from = NULL;
 	GtkWidget *scrolledwin_quotefmt;
 	GtkWidget *text_fw_quotefmt;
 
@@ -312,24 +379,38 @@ void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 	g_return_if_fail(parent_box != NULL);
 	if (checkbtn_forward_with_format) {
 		g_return_if_fail(checkbtn_forward_with_format != NULL);
-		g_return_if_fail(checkbtn_forward_text != NULL);
 	}
 	g_return_if_fail(edit_fw_quotemark != NULL);
 	g_return_if_fail(edit_fw_format != NULL);
 
 	if (checkbtn_forward_with_format)
-		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format, checkbtn_forward_text);
+		PACK_CHECK_BUTTON (parent_box, checkbtn_use_format,
+				   _("Use template when forwarding messages"));
 
-	PACK_FRAME (parent_box, frame_quote, _("Forward format"));
-
-	if (checkbtn_forward_with_format)
-		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, frame_quote);
-
-	vbox_quote = gtk_vbox_new (FALSE, VSPACING_NARROW);
-	gtk_widget_show (vbox_quote);
-	gtk_container_add (GTK_CONTAINER (frame_quote), vbox_quote);
+	vbox_quote = gtk_vbox_new (FALSE, 4);
+	gtk_widget_show(vbox_quote);
+	gtk_container_add(GTK_CONTAINER (parent_box), vbox_quote);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);
 
+	if (override_from_format) {
+		hbox3 = gtk_hbox_new (FALSE, 8);
+		gtk_widget_show (hbox3);
+		gtk_box_pack_start (GTK_BOX (vbox_quote), hbox3, FALSE, FALSE, 0);
+
+		label_from = gtk_label_new (_("From"));
+		gtk_widget_show (label_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), label_from, FALSE, FALSE, 0);
+
+		entry_from = gtk_entry_new ();
+		gtk_widget_show (entry_from);
+		gtk_box_pack_start (GTK_BOX (hbox3), entry_from, TRUE, TRUE, 0);
+		gtk_widget_set_size_request (entry_from, 100, -1);
+
+		gtk_tooltips_set_tip(gtk_tooltips_new(), entry_from,
+				_("Override From header. This doesn't change the account used to forward."),
+				NULL);
+	}
+
 	hbox1 = gtk_hbox_new (FALSE, 32);
 	gtk_widget_show (hbox1);
 	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, FALSE, FALSE, 0);
@@ -375,12 +456,24 @@ void quotefmt_create_forward_fmt_widgets(GtkWindow *parent_window,
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (text_fw_quotefmt), TRUE);
 	gtk_widget_set_size_request (text_fw_quotefmt, -1, 100);
 
+	if (checkbtn_forward_with_format) {
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, label_quotemark);
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_fw_quotemark);
+		if (override_from_format)
+			SET_TOGGLE_SENSITIVITY(checkbtn_use_format, entry_from);
+		SET_TOGGLE_SENSITIVITY(checkbtn_use_format, text_fw_quotefmt);
+	}
+
 	if (add_info_button)
 		quotefmt_add_info_button(parent_window, vbox_quote);
+	if (set_defaults_func)
+		quotefmt_add_defaults_button(parent_window, vbox_quote, set_defaults_func);
 
 	if (checkbtn_forward_with_format)
 		*checkbtn_forward_with_format = checkbtn_use_format;
 	*edit_fw_quotemark = entry_fw_quotemark;
+	if (override_from_format)
+		*override_from_format = entry_from;
 	*edit_fw_format = text_fw_quotefmt;
 }
 
@@ -391,7 +484,8 @@ void quotefmt_add_info_button(GtkWindow *parent_window, GtkWidget *parent_box)
 
 	hbox_formatdesc = gtk_hbox_new (FALSE, 32);
 	gtk_widget_show (hbox_formatdesc);
-	gtk_box_pack_start (GTK_BOX (parent_box), hbox_formatdesc, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (parent_box), hbox_formatdesc, FALSE, FALSE,
+				VBOX_BORDER);
 
 #if GTK_CHECK_VERSION(2, 8, 0)
 	btn_formatdesc = gtk_button_new_from_stock(GTK_STOCK_INFO);
@@ -405,57 +499,85 @@ void quotefmt_add_info_button(GtkWindow *parent_window, GtkWidget *parent_box)
 			 G_CALLBACK(quote_fmt_quote_description), GTK_WIDGET(parent_window));
 }
 
+void quotefmt_add_defaults_button(GtkWindow *parent_window,
+								  GtkWidget *parent_box,
+								  void(*set_defaults_func)(void))
+{
+	GtkWidget *hbox_formatdesc;
+	GtkWidget *btn_formatdesc;
+
+	g_return_if_fail(set_defaults_func != NULL);
+
+	hbox_formatdesc = gtk_hbox_new (FALSE, 32);
+	gtk_widget_show (hbox_formatdesc);
+	gtk_box_pack_start (GTK_BOX (parent_box), hbox_formatdesc, FALSE, FALSE,
+				VBOX_BORDER);
+
+	btn_formatdesc = gtk_button_new_with_mnemonic (_("Defaults"));
+	gtk_button_set_image (GTK_BUTTON(btn_formatdesc),
+		gtk_image_new_from_stock(GTK_STOCK_UNDO, GTK_ICON_SIZE_BUTTON));
+	gtk_widget_show (btn_formatdesc);
+	gtk_box_pack_start (GTK_BOX (hbox_formatdesc), btn_formatdesc, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(btn_formatdesc), "clicked",
+			 G_CALLBACK(set_defaults_func), GTK_WIDGET(parent_window));
+}
 
 void quotefmt_check_new_msg_formats(gboolean use_format,
+									gchar *override_from_fmt,
 									gchar *subject_fmt,
 									gchar *body_fmt)
 {
 	if (use_format) {
 		gint line;
 
-		if (!prefs_template_string_is_valid(subject_fmt, NULL))
+		if (override_from_fmt && !prefs_template_string_is_valid(override_from_fmt, NULL, FALSE, TRUE))
+			alertpanel_error(_("New message From format error."));
+
+		if (!prefs_template_string_is_valid(subject_fmt, NULL, FALSE, FALSE))
 			alertpanel_error(_("New message subject format error."));
 
-		if (!prefs_template_string_is_valid(body_fmt, &line)) {
-			gchar *msg = g_strdup_printf(_("New message body format error at line %d."), line);
-			alertpanel_error(msg);
-			g_free(msg);
+		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE, FALSE)) {
+			alertpanel_error(_("New message body format error at line %d."), line);
 		}
 	}
 }
 
 void quotefmt_check_reply_formats(gboolean use_format,
+									gchar *override_from_fmt,
 									gchar *quotation_mark,
 									gchar *body_fmt)
 {
 	if (use_format) {
 		gint line;
 
-		if (!prefs_template_string_is_valid(quotation_mark, NULL))
+		if (!prefs_template_string_is_valid(quotation_mark, NULL, FALSE, FALSE))
 			alertpanel_error(_("Message reply quotation mark format error."));
 
-		if (!prefs_template_string_is_valid(body_fmt, &line)) {
-			gchar *msg = g_strdup_printf(_("Message reply format error at line %d."), line);
-			alertpanel_error(msg);
-			g_free(msg);
+		if (override_from_fmt && !prefs_template_string_is_valid(override_from_fmt, NULL, FALSE, TRUE))
+			alertpanel_error(_("Message reply From format error."));
+
+		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE, FALSE)) {
+			alertpanel_error(_("Message reply format error at line %d."), line);
 		}
 	}
 }
 
 void quotefmt_check_forward_formats(gboolean use_format,
+									gchar *override_from_fmt,
 									gchar *quotation_mark,
 									gchar *body_fmt)
 {
 	if (use_format) {
 		gint line;
 
-		if (!prefs_template_string_is_valid(quotation_mark, NULL))
+		if (!prefs_template_string_is_valid(quotation_mark, NULL, FALSE, FALSE))
 			alertpanel_error(_("Message forward quotation mark format error."));
 
-		if (!prefs_template_string_is_valid(body_fmt, &line)) {
-			gchar *msg = g_strdup_printf(_("Message forward format error at line %d."), line);
-			alertpanel_error(msg);
-			g_free(msg);
+		if (override_from_fmt && !prefs_template_string_is_valid(override_from_fmt, NULL, FALSE, TRUE))
+			alertpanel_error(_("Message forward From format error."));
+
+		if (!prefs_template_string_is_valid(body_fmt, &line, TRUE, FALSE)) {
+			alertpanel_error(_("Message forward format error at line %d."), line);
 		}
 	}
 }
