@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -68,6 +68,8 @@
 #include "base64.h"
 #include "inputdialog.h"
 #include "timing.h"
+#include "tags.h"
+#include "addrindex.h"
 
 static GdkColor quote_colors[3] = {
 	{(gulong)0, (gushort)0, (gushort)0, (gushort)0},
@@ -98,7 +100,7 @@ static GdkColor emphasis_color = {
 	(gulong)0,
 	(gushort)0,
 	(gushort)0,
-	(gushort)0xcfff
+	(gushort)0
 };
 
 static GdkCursor *hand_cursor = NULL;
@@ -206,6 +208,7 @@ static void save_file_cb			(TextView 	*textview,
 static void open_image_cb			(TextView 	*textview,
 						 guint		 action,
 						 void		*data);
+static void textview_show_tags(TextView *textview);
 
 static GtkItemFactoryEntry textview_link_popup_entries[] = 
 {
@@ -358,17 +361,33 @@ static void textview_create_tags(GtkTextView *text, TextView *textview)
 {
 	GtkTextBuffer *buffer;
 	GtkTextTag *tag, *qtag;
+	static GdkColor yellow, black;
+	static gboolean color_init = FALSE;
 	static PangoFontDescription *font_desc, *bold_font_desc;
 	
+	if (!color_init) {
+		gdk_color_parse("#f5f6be", &yellow);
+		gdk_color_parse("#000000", &black);
+		color_init = gdk_colormap_alloc_color(
+			gdk_colormap_get_system(), &yellow, FALSE, TRUE);
+		color_init &= gdk_colormap_alloc_color(
+			gdk_colormap_get_system(), &black, FALSE, TRUE);
+	}
+
 	if (!font_desc)
 		font_desc = pango_font_description_from_string
 			(NORMAL_FONT);
 
 	if (!bold_font_desc) {
-		bold_font_desc = pango_font_description_from_string
-			(NORMAL_FONT);
-		pango_font_description_set_weight
-			(bold_font_desc, PANGO_WEIGHT_BOLD);
+		if (prefs_common.derive_from_normal_font || !BOLD_FONT) {
+			bold_font_desc = pango_font_description_from_string
+				(NORMAL_FONT);
+			pango_font_description_set_weight
+				(bold_font_desc, PANGO_WEIGHT_BOLD);
+		} else {
+			bold_font_desc = pango_font_description_from_string
+				(BOLD_FONT);
+		}
 	}
 
 	buffer = gtk_text_view_get_buffer(text);
@@ -421,6 +440,16 @@ static void textview_create_tags(GtkTextView *text, TextView *textview)
 				"foreground-gdk", &quote_colors[2],
 				NULL);
 	}
+#if GTK_CHECK_VERSION(2, 8, 0)
+	gtk_text_buffer_create_tag(buffer, "tags",
+			"foreground-gdk", &black,
+			"paragraph-background-gdk", &yellow,
+			NULL);
+#else
+	gtk_text_buffer_create_tag(buffer, "tags",
+			"foreground-gdk", &emphasis_color,
+			NULL);
+#endif
 	gtk_text_buffer_create_tag(buffer, "emphasis",
 			"foreground-gdk", &emphasis_color,
 			NULL);
@@ -439,6 +468,10 @@ static void textview_create_tags(GtkTextView *text, TextView *textview)
                          G_CALLBACK(textview_uri_button_pressed), textview);
 	g_signal_connect(G_OBJECT(tag), "event",
                          G_CALLBACK(textview_uri_button_pressed), textview);
+/*	if (font_desc)
+		pango_font_description_free(font_desc);
+	if (bold_font_desc)
+		pango_font_description_free(bold_font_desc);*/
  }
 
 void textview_init(TextView *textview)
@@ -473,7 +506,6 @@ void textview_init(TextView *textview)
 static void textview_update_message_colors(TextView *textview)
 {
 	GdkColor black = {0, 0, 0, 0};
-	GdkColor colored_emphasis = {0, 0, 0, 0xcfff};
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
 
 	GtkTextTagTable *tags = gtk_text_buffer_get_tag_table(buffer);
@@ -495,7 +527,8 @@ static void textview_update_message_colors(TextView *textview)
 					       &uri_color);
 		gtkut_convert_int_to_gdk_color(prefs_common.signature_col,
 					       &signature_color);
-		emphasis_color = colored_emphasis;
+		gtkut_convert_int_to_gdk_color(prefs_common.emphasis_col,
+					       &emphasis_color);
 	}
 	if (prefs_common.enable_color && prefs_common.enable_bgcolor) {
 		gtkut_convert_int_to_gdk_color(prefs_common.quote_level1_bgcol,
@@ -528,22 +561,6 @@ void textview_reflect_prefs(TextView *textview)
 					 prefs_common.textview_cursor_visible);
 }
 
-void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
-			   const gchar *file)
-{
-	textview->loading = TRUE;
-	textview->stop_loading = FALSE;
-
-	textview_clear(textview);
-
-	textview_add_parts(textview, mimeinfo);
-
-	textview_set_position(textview, 0);
-
-	textview->loading = FALSE;
-	textview->stop_loading = FALSE;
-}
-
 void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 {
 	START_TIMING("");
@@ -611,13 +628,23 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 
 	if ((mimeinfo->type == MIMETYPE_MESSAGE) && !g_ascii_strcasecmp(mimeinfo->subtype, "rfc822")) {
 		FILE *fp;
-
-		fp = g_fopen(mimeinfo->data.filename, "rb");
+		if (mimeinfo->content == MIMECONTENT_MEM)
+			fp = str_open_as_stream(mimeinfo->data.mem);
+		else
+			fp = g_fopen(mimeinfo->data.filename, "rb");
+		if (!fp) {
+			FILE_OP_ERROR(mimeinfo->data.filename, "fopen");
+			END_TIMING();
+			return;
+		}
 		fseek(fp, mimeinfo->offset, SEEK_SET);
 		headers = textview_scan_header(textview, fp);
 		if (headers) {
 			if (charcount > 0)
 				gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+			
+			if (procmime_mimeinfo_parent(mimeinfo) == NULL)
+				textview_show_tags(textview);
 			textview_show_header(textview, headers);
 			procheader_header_array_destroy(headers);
 		}
@@ -649,6 +676,8 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 		if (mimeinfo->type == MIMETYPE_IMAGE  &&
 		    prefs_common.inline_img ) {
 			GdkPixbuf *pixbuf;
+			gint avail_width;
+			gint avail_height;
 			GError *error = NULL;
 			gchar *filename;
 			ClickableText *uri;
@@ -670,11 +699,12 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 			} else {
 				gint w, h;
 				gdk_pixbuf_get_file_info(filename, &w, &h);
-				if (textview->scrolledwin->allocation.width - 100 > 0 &&
-				    w > textview->scrolledwin->allocation.width - 100)
+				avail_width = textview->scrolledwin->allocation.width;
+				avail_height = textview->scrolledwin->allocation.height;
+				if (avail_width - 100 > 0 &&
+				    (w > avail_width || h > avail_height))
 					pixbuf = gdk_pixbuf_new_from_file_at_scale(filename, 
-						textview->scrolledwin->allocation.width - 100, 
-						-1, TRUE, &error);
+						avail_width, avail_height, TRUE, &error);
 				else
 					pixbuf = gdk_pixbuf_new_from_file(filename, &error);
 			}
@@ -734,7 +764,9 @@ static void recursive_add_parts(TextView *textview, GNode *node)
         mimeinfo = (MimeInfo *) node->data;
         
         textview_add_part(textview, mimeinfo);
-        
+#ifdef MAEMO
+	textview_set_position(textview, 0);
+#endif        
         if ((mimeinfo->type != MIMETYPE_MULTIPART) &&
             (mimeinfo->type != MIMETYPE_MESSAGE)) {
 	    	END_TIMING();
@@ -822,6 +854,8 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	GtkTextView *text;
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
+	const gchar *name;
+	gchar *content_type;
 
 	if (!partinfo) return;
 
@@ -833,23 +867,57 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	gtk_text_buffer_get_start_iter(buffer, &iter);
 
 	TEXTVIEW_INSERT("\n");
-	TEXTVIEW_INSERT(_("  The following can be performed on this part by\n"));
-	TEXTVIEW_INSERT(_("  right-clicking the icon or list item:\n"));
+
+	name = procmime_mimeinfo_get_parameter(partinfo, "filename");
+	if (name == NULL)
+		name = procmime_mimeinfo_get_parameter(partinfo, "name");
+	if (name != NULL) {
+		content_type = procmime_get_content_type_str(partinfo->type,
+						     partinfo->subtype);
+		TEXTVIEW_INSERT("  ");
+		TEXTVIEW_INSERT_BOLD(name);
+		TEXTVIEW_INSERT(" (");
+		TEXTVIEW_INSERT(content_type);
+		TEXTVIEW_INSERT(", ");
+		TEXTVIEW_INSERT(to_human_readable(partinfo->length));
+		TEXTVIEW_INSERT("):\n\n");
+		
+		g_free(content_type);
+	}
+	TEXTVIEW_INSERT(_("  The following can be performed on this part\n"));
+#ifndef MAEMO
+	TEXTVIEW_INSERT(_("  by right-clicking the icon or list item:"));
+#endif
+	TEXTVIEW_INSERT("\n");
 
 	TEXTVIEW_INSERT(_("     - To save, select "));
 	TEXTVIEW_INSERT_LINK(_("'Save as...'"), "sc://save_as", NULL);
-	TEXTVIEW_INSERT(_(" (Shortcut key: 'y')\n"));
+#ifndef MAEMO
+	TEXTVIEW_INSERT(_(" (Shortcut key: 'y')"));
+#endif
+	TEXTVIEW_INSERT("\n");
+
 	TEXTVIEW_INSERT(_("     - To display as text, select "));
 	TEXTVIEW_INSERT_LINK(_("'Display as text'"), "sc://display_as_text", NULL);
-	TEXTVIEW_INSERT(_(" (Shortcut key: 't')\n"));
+
+#ifndef MAEMO
+	TEXTVIEW_INSERT(_(" (Shortcut key: 't')"));
+#endif
+	TEXTVIEW_INSERT("\n");
+
 	TEXTVIEW_INSERT(_("     - To open with an external program, select "));
 	TEXTVIEW_INSERT_LINK(_("'Open'"), "sc://open", NULL);
+
+#ifndef MAEMO
 	TEXTVIEW_INSERT(_(" (Shortcut key: 'l')\n"));
 	TEXTVIEW_INSERT(_("       (alternately double-click, or click the middle "));
 	TEXTVIEW_INSERT(_("mouse button)\n"));
 	TEXTVIEW_INSERT(_("     - Or use "));
 	TEXTVIEW_INSERT_LINK(_("'Open with...'"), "sc://open_with", NULL);
-	TEXTVIEW_INSERT(_(" (Shortcut key: 'o')\n"));
+	TEXTVIEW_INSERT(_(" (Shortcut key: 'o')"));
+#endif
+	TEXTVIEW_INSERT("\n");
+
 	textview_show_icon(textview, GTK_STOCK_DIALOG_INFO);
 }
 
@@ -860,8 +928,9 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 	CodeConverter *conv;
 	const gchar *charset, *p, *cmd;
 	GSList *cur;
-	int lines = 0;
-	
+	gboolean continue_write = TRUE;
+	size_t wrote = 0, i = 0;
+
 	if (textview->messageview->forced_charset)
 		charset = textview->messageview->forced_charset;
 	else
@@ -938,7 +1007,7 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 			dup(pfd[1]);
 			rc = execvp(argv[0], argv);
 			close(pfd[1]);
-			printf (_("The command to view attachment "
+			g_print(_("The command to view attachment "
 			        "as text failed:\n"
 			        "    %s\n"
 			        "Exit code %d\n"), buf, rc);
@@ -949,9 +1018,6 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 		while (fgets(buf, sizeof(buf), tmpfp)) {
 			textview_write_line(textview, buf, conv, TRUE);
 			
-			lines++;
-			if (lines % 500 == 0)
-				GTK_EVENTS_FLUSH();
 			if (textview->stop_loading) {
 				fclose(tmpfp);
 				waitpid(pid, pfd, 0);
@@ -966,20 +1032,30 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 #endif
 	} else {
 textview_default:
-		lines = 0;
-		tmpfp = g_fopen(mimeinfo->data.filename, "rb");
+		if (mimeinfo->content == MIMECONTENT_MEM)
+			tmpfp = str_open_as_stream(mimeinfo->data.mem);
+		else
+			tmpfp = g_fopen(mimeinfo->data.filename, "rb");
+		if (!tmpfp) {
+			FILE_OP_ERROR(mimeinfo->data.filename, "fopen");
+			return;
+		}
 		fseek(tmpfp, mimeinfo->offset, SEEK_SET);
 		debug_print("Viewing text content of type: %s (length: %d)\n", mimeinfo->subtype, mimeinfo->length);
-		while ((ftell(tmpfp) < mimeinfo->offset + mimeinfo->length) &&
-		       (fgets(buf, sizeof(buf), tmpfp) != NULL)) {
+		while (((i = ftell(tmpfp)) < mimeinfo->offset + mimeinfo->length) &&
+		       (fgets(buf, sizeof(buf), tmpfp) != NULL)
+		       && continue_write) {
 			textview_write_line(textview, buf, conv, TRUE);
-			lines++;
-			if (lines % 500 == 0)
-				GTK_EVENTS_FLUSH();
 			if (textview->stop_loading) {
 				fclose(tmpfp);
 				return;
 			}
+			wrote += ftell(tmpfp)-i;
+			if (mimeinfo->length > 1024*1024 
+			&&  wrote > 1024*1024
+			&& !textview->messageview->show_full_text) {
+				continue_write = FALSE;
+			}
 		}
 		fclose(tmpfp);
 	}
@@ -987,7 +1063,6 @@ textview_default:
 	conv_code_converter_destroy(conv);
 	procmime_force_encoding(0);
 
-	lines = 0;
 	textview->uri_list = g_slist_reverse(textview->uri_list);
 	for (cur = textview->uri_list; cur; cur = cur->next) {
 		ClickableText *uri = (ClickableText *)cur->data;
@@ -996,14 +1071,19 @@ textview_default:
 		if (!prefs_common.hide_quotes ||
 		    uri->quote_level+1 < prefs_common.hide_quotes) {
 			textview_toggle_quote(textview, cur, uri, TRUE);
-			lines++;
-			if (lines % 500 == 0)
-				GTK_EVENTS_FLUSH();
 			if (textview->stop_loading) {
 				return;
 			}
 		}
 	}
+	
+	if (continue_write == FALSE) {
+		messageview_show_partial_display(
+			textview->messageview, 
+			textview->messageview->msginfo,
+			mimeinfo->length);
+	}
+	GTK_EVENTS_FLUSH();
 }
 
 static void textview_show_html(TextView *textview, FILE *fp,
@@ -1436,7 +1516,7 @@ do_quote:
 			if (textview->uri_list) {
 				lasturi = (ClickableText *)last->data;
 			} else {
-				printf("oops (%d %d)\n",
+				g_print("oops (%d %d)\n",
 					real_quotelevel, textview->prev_quote_level);
 			}		
 			if (lasturi->is_quote == FALSE) {
@@ -1577,25 +1657,29 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 	GtkTextTag *tag;
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
 	GtkTextTagTable *tags = gtk_text_buffer_get_tag_table(buffer);
-	
-	if (NORMAL_FONT) {
-		PangoFontDescription *font_desc, *bold_font_desc;
-		font_desc = pango_font_description_from_string
-						(NORMAL_FONT);
+	PangoFontDescription *font_desc, *bold_font_desc;
+
+	font_desc = pango_font_description_from_string
+					(NORMAL_FONT);
+	if (font_desc) {
+		gtk_widget_modify_font(textview->text, font_desc);
+		CHANGE_TAG_FONT("header", font_desc);
+		CHANGE_TAG_FONT("hlink", font_desc);
+		pango_font_description_free(font_desc);
+	}
+	if (prefs_common.derive_from_normal_font || !BOLD_FONT) {
 		bold_font_desc = pango_font_description_from_string
 						(NORMAL_FONT);
-		if (font_desc) {
-			gtk_widget_modify_font(textview->text, font_desc);
-			CHANGE_TAG_FONT("header", font_desc);
-			CHANGE_TAG_FONT("hlink", font_desc);
-			pango_font_description_free(font_desc);
-		}
-		if (bold_font_desc) {
+		if (bold_font_desc)
 			pango_font_description_set_weight
 				(bold_font_desc, PANGO_WEIGHT_BOLD);
-			CHANGE_TAG_FONT("header_title", bold_font_desc);
-			pango_font_description_free(bold_font_desc);
-		}
+	} else {
+		bold_font_desc = pango_font_description_from_string
+						(BOLD_FONT);
+	}
+	if (bold_font_desc) {
+		CHANGE_TAG_FONT("header_title", bold_font_desc);
+		pango_font_description_free(bold_font_desc);
 	}
 
 	if (prefs_common.textfont) {
@@ -1856,6 +1940,177 @@ bail:
 }
 #endif
 
+static void textview_save_contact_pic(TextView *textview)
+{
+	MsgInfo *msginfo = textview->messageview->msginfo;
+	gchar *filename = NULL;
+	GError *error = NULL;
+	GdkPixbuf *picture = NULL;
+				
+	if (!msginfo->extradata || (!msginfo->extradata->face && !msginfo->extradata->xface))
+		return;
+
+	if (textview->image) 
+		picture = gtk_image_get_pixbuf(GTK_IMAGE(textview->image));
+
+	filename = addrindex_get_picture_file(msginfo->from);
+	if (!filename)
+		return;
+	if (!is_file_exist(filename)) {
+		gdk_pixbuf_save(picture, filename, "png", &error, NULL);
+		if (error) {
+			g_warning(_("Failed to save image: \n%s"),
+					error->message);
+			g_error_free(error);
+		}
+	}
+	g_free(filename);
+}
+
+static void textview_show_contact_pic(TextView *textview)
+{
+	MsgInfo *msginfo = textview->messageview->msginfo;
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	int x = 0;
+	gchar *filename = NULL;
+	GError *error = NULL;
+	GdkPixbuf *picture = NULL;
+	gint w, h;
+				
+	if (prefs_common.display_header_pane
+	||  !prefs_common.display_xface)
+		goto bail;
+	
+	if (msginfo->extradata && (msginfo->extradata->face || msginfo->extradata->xface))
+		return;
+
+	if (textview->image) 
+		gtk_widget_destroy(textview->image);
+
+	filename = addrindex_get_picture_file(msginfo->from);
+	
+	if (!filename)
+		goto bail;
+	if (!is_file_exist(filename)) {
+		g_free(filename);
+		goto bail;
+	}
+
+	gdk_pixbuf_get_file_info(filename, &w, &h);
+	
+	if (w > 48 || h > 48)
+		picture = gdk_pixbuf_new_from_file_at_scale(filename, 
+						48, 48, TRUE, &error);
+	else
+		picture = gdk_pixbuf_new_from_file(filename, &error);
+
+	if (error) {
+		debug_print("Failed to import image: \n%s",
+				error->message);
+		g_error_free(error);
+		goto bail;
+	}
+	g_free(filename);
+
+	if (picture) {
+		textview->image = gtk_image_new_from_pixbuf(picture);
+		g_object_unref(picture);
+	}
+	g_return_if_fail(textview->image != NULL);
+
+	gtk_widget_show(textview->image);
+	
+	x = textview->text->allocation.width - WIDTH -5;
+
+	gtk_text_view_add_child_in_window(text, textview->image, 
+		GTK_TEXT_WINDOW_TEXT, x, 5);
+
+	gtk_widget_show_all(textview->text);
+	
+	return;
+bail:
+	if (textview->image) 
+		gtk_widget_destroy(textview->image);
+	textview->image = NULL;
+	
+}
+
+static gint textview_tag_cmp_list(gconstpointer a, gconstpointer b)
+{
+	gint id_a = GPOINTER_TO_INT(a);
+	gint id_b = GPOINTER_TO_INT(b);
+	const gchar *tag_a = tags_get_tag(id_a);
+	const gchar *tag_b = tags_get_tag(id_b);
+	
+	if (tag_a == NULL)
+		return tag_b == NULL ? 0:1;
+	
+	if (tag_b == NULL)
+		return tag_a == NULL ? 0:1;
+
+	return g_utf8_collate(tag_a, tag_b);
+}
+
+
+static void textview_show_tags(TextView *textview)
+{
+	MsgInfo *msginfo = textview->messageview->msginfo;
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer(text);
+	GtkTextIter iter;
+	ClickableText *uri;
+	GSList *cur, *orig;
+	gboolean found_tag = FALSE;
+	
+	if (!msginfo->tags)
+		return;
+	
+	cur = orig = g_slist_sort(g_slist_copy(msginfo->tags), textview_tag_cmp_list);
+
+	for (; cur; cur = cur->next) {
+		if (tags_get_tag(GPOINTER_TO_INT(cur->data)) != NULL) {
+			found_tag = TRUE;
+			break;
+		}
+	}
+	if (!found_tag) {
+		g_slist_free(orig);
+		return;
+	}
+
+	gtk_text_buffer_get_end_iter (buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name(buffer,
+		&iter, _("Tags: "), -1,
+		"header_title", "header", "tags", NULL);
+
+	for (cur = orig; cur; cur = cur->next) {
+		const gchar *cur_tag = tags_get_tag(GPOINTER_TO_INT(cur->data));
+		if (!cur_tag)
+			continue;
+		uri = g_new0(ClickableText, 1);
+		uri->uri = g_strdup("");
+		uri->start = gtk_text_iter_get_offset(&iter);
+		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
+			cur_tag, -1,
+			"link", "header", "tags", NULL);
+		uri->end = gtk_text_iter_get_offset(&iter);
+		uri->filename = g_strdup_printf("sc://search_tags:%s", cur_tag);
+		uri->data = NULL;
+		textview->uri_list =
+			g_slist_prepend(textview->uri_list, uri);
+		if (cur->next && tags_get_tag(GPOINTER_TO_INT(cur->next->data)))
+			gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, ", ", 2,
+				"header", "tags", NULL);
+		else
+			gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, " ", 1,
+				"header", "tags", NULL);
+	}
+	g_slist_free(orig);
+
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, "\n", 1,
+		"header", "tags", NULL);
+}
+
 static void textview_show_header(TextView *textview, GPtrArray *headers)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
@@ -1926,6 +2181,8 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 #if HAVE_LIBCOMPFACE
 	textview_show_xface(textview);
 #endif
+	textview_save_contact_pic(textview);
+	textview_show_contact_pic(textview);
 }
 
 gboolean textview_search_string(TextView *textview, const gchar *str,
@@ -2012,6 +2269,7 @@ static gint textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	case GDK_y:
 	case GDK_t:
 	case GDK_l:
+	case GDK_o:
 	case GDK_c:
 	case GDK_a:
 		if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) == 0) {
@@ -2429,7 +2687,7 @@ static gboolean textview_uri_button_pressed(GtkTextTag *tag, GObject *obj,
 			if (bevent->button == 1 &&
 			    textview_uri_security_check(textview, uri) == TRUE) 
 					open_uri(uri->uri,
-						 prefs_common.uri_cmd);
+						 prefs_common_get_uri_cmd());
 			else if (bevent->button == 3 && !qlink) {
 				g_object_set_data(
 					G_OBJECT(textview->link_popup_menu),
@@ -2557,7 +2815,7 @@ static void open_uri_cb (TextView *textview, guint action, void *data)
 
 	if (textview_uri_security_check(textview, uri) == TRUE) 
 		open_uri(uri->uri,
-			 prefs_common.uri_cmd);
+			 prefs_common_get_uri_cmd());
 	g_object_set_data(G_OBJECT(textview->link_popup_menu), "menu_button",
 			  NULL);
 }
@@ -2602,7 +2860,7 @@ static void open_image_cb (TextView *textview, guint action, void *data)
 			   "('%s' will be replaced with file name)"),
 			 prefs_common.mime_open_cmd,
 			 prefs_common.mime_open_cmd_history,
-			 TRUE, &remember);
+			 &remember);
 		if (cmd && remember) {
 			mailcap_update_default("image/jpeg", cmd);
 		}
@@ -2710,17 +2968,42 @@ static void add_uri_to_addrbook_cb (TextView *textview, guint action, void *data
 	gchar *fromname, *fromaddress;
 	ClickableText *uri = g_object_get_data(G_OBJECT(textview->mail_popup_menu),
 					   "menu_button");
+	GtkWidget *image = NULL;
+	GdkPixbuf *picture = NULL;
+	gboolean use_picture = FALSE;
 	if (uri == NULL)
 		return;
 
 	/* extract url */
 	fromaddress = g_strdup(uri->uri + 7);
-	/* Hiroyuki: please put this function in utils.c! */
+	
+	if (textview->messageview->msginfo &&
+	   !strcmp2(fromaddress, textview->messageview->msginfo->from))
+		use_picture = TRUE;
+
 	fromname = procheader_get_fromname(fromaddress);
 	extract_address(fromaddress);
 
-	/* Add to address book - Match */
-	addressbook_add_contact( fromname, fromaddress, NULL );
+	if (use_picture && 
+	    textview->messageview->msginfo &&
+	    textview->messageview->msginfo->extradata &&
+	    textview->messageview->msginfo->extradata->face) {
+		image = face_get_from_header(textview->messageview->msginfo->extradata->face);
+	}
+#if HAVE_LIBCOMPFACE 
+	else if (use_picture && 
+	         textview->messageview->msginfo &&
+	         textview->messageview->msginfo->extradata &&
+		 textview->messageview->msginfo->extradata->xface) {
+		image = xface_get_from_header(textview->messageview->msginfo->extradata->xface,
+				&textview->text->style->white,
+				mainwindow_get_mainwindow()->window->window);	
+	}
+#endif
+	if (image)
+		picture = gtk_image_get_pixbuf(GTK_IMAGE(image));
+
+	addressbook_add_contact( fromname, fromaddress, NULL, picture);
 
 	g_free(fromaddress);
 	g_free(fromname);
@@ -2741,8 +3024,11 @@ static void mail_to_uri_cb (TextView *textview, guint action, void *data)
 		folder_item = textview->messageview->msginfo->folder;
 		if (folder_item->prefs && folder_item->prefs->enable_default_account)
 			account = account_find_from_id(folder_item->prefs->default_account);
+		
+		compose_new_with_folderitem(account, folder_item, uri->uri+7);
+	} else {
+		compose_new(account, uri->uri + 7, NULL);
 	}
-	compose_new(account, uri->uri + 7, NULL);
 }
 
 static void copy_mail_to_uri_cb	(TextView *textview, guint action, void *data)
